# coding: utf-8

# flake8: noqa
"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

# import models into model package
from openapi_client.models.aws_config import AWSConfig
from openapi_client.models.aws_memory_db_cluster_config import AWSMemoryDBClusterConfig
from openapi_client.models.access_config import AccessConfig
from openapi_client.models.actor_exit_detail_group_aggregate import ActorExitDetailGroupAggregate
from openapi_client.models.actor_exit_detail_group_aggregate_response import ActorExitDetailGroupAggregateResponse
from openapi_client.models.actor_grouped_aggregate_metrics import ActorGroupedAggregateMetrics
from openapi_client.models.actor_job_group_aggregate import ActorJobGroupAggregate
from openapi_client.models.actor_job_group_aggregate_response import ActorJobGroupAggregateResponse
from openapi_client.models.actor_name_group_aggregate import ActorNameGroupAggregate
from openapi_client.models.actor_name_group_aggregate_response import ActorNameGroupAggregateResponse
from openapi_client.models.actor_state import ActorState
from openapi_client.models.actor_status import ActorStatus
from openapi_client.models.actor_summary import ActorSummary
from openapi_client.models.actor_table_config import ActorTableConfig
from openapi_client.models.actor_table_response import ActorTableResponse
from openapi_client.models.actor_table_row import ActorTableRow
from openapi_client.models.actorsummary_response import ActorsummaryResponse
from openapi_client.models.admin_create_user import AdminCreateUser
from openapi_client.models.admin_created_user import AdminCreatedUser
from openapi_client.models.admincreateduser_list_response import AdmincreateduserListResponse
from openapi_client.models.aggregated_usage import AggregatedUsage
from openapi_client.models.aggregated_usage_query import AggregatedUsageQuery
from openapi_client.models.aggregatedusage_response import AggregatedusageResponse
from openapi_client.models.aioa_cloud_waitlist_record import AioaCloudWaitlistRecord
from openapi_client.models.aioacloudwaitlistrecord_response import AioacloudwaitlistrecordResponse
from openapi_client.models.alert_issue_type import AlertIssueType
from openapi_client.models.anyscale_aws_account import AnyscaleAWSAccount
from openapi_client.models.anyscale_service_account import AnyscaleServiceAccount
from openapi_client.models.anyscaleawsaccount_response import AnyscaleawsaccountResponse
from openapi_client.models.anyscaled_credential_response import AnyscaledCredentialResponse
from openapi_client.models.anyscaledcredentialresponse_response import AnyscaledcredentialresponseResponse
from openapi_client.models.anyscaleserviceaccount_response import AnyscaleserviceaccountResponse
from openapi_client.models.api_key_info import ApiKeyInfo
from openapi_client.models.api_key_parameters import ApiKeyParameters
from openapi_client.models.apikeyinfo_list_response import ApikeyinfoListResponse
from openapi_client.models.app_config import AppConfig
from openapi_client.models.app_config_config_schema import AppConfigConfigSchema
from openapi_client.models.appconfig_list_response import AppconfigListResponse
from openapi_client.models.appconfig_response import AppconfigResponse
from openapi_client.models.application_type import ApplicationType
from openapi_client.models.applied_snapshot import AppliedSnapshot
from openapi_client.models.apply_autoscaling_config_update_model import ApplyAutoscalingConfigUpdateModel
from openapi_client.models.apply_multi_version_update_weights_update_model import ApplyMultiVersionUpdateWeightsUpdateModel
from openapi_client.models.apply_production_service_multi_version_v2_model import ApplyProductionServiceMultiVersionV2Model
from openapi_client.models.apply_production_service_v2_model import ApplyProductionServiceV2Model
from openapi_client.models.apply_version_weight_update_model import ApplyVersionWeightUpdateModel
from openapi_client.models.archive_status import ArchiveStatus
from openapi_client.models.attach_machine_pool_to_cloud_request import AttachMachinePoolToCloudRequest
from openapi_client.models.attachmachinepooltocloudresponse_response import AttachmachinepooltocloudresponseResponse
from openapi_client.models.auth_resource_type import AuthResourceType
from openapi_client.models.aws_region_and_zones import AwsRegionAndZones
from openapi_client.models.aws_region_info import AwsRegionInfo
from openapi_client.models.awsregionandzones_response import AwsregionandzonesResponse
from openapi_client.models.azure_config import AzureConfig
from openapi_client.models.azure_operator_audience import AzureOperatorAudience
from openapi_client.models.azureoperatoraudience_response import AzureoperatoraudienceResponse
from openapi_client.models.baseimagesenum import BASEIMAGESENUM
from openapi_client.models.backend_server_api_product_models_catalog_client_models_table_metadata import BackendServerApiProductModelsCatalogClientModelsTableMetadata
from openapi_client.models.backend_server_api_product_models_data_catalogs_table_metadata import BackendServerApiProductModelsDataCatalogsTableMetadata
from openapi_client.models.base_job_status import BaseJobStatus
from openapi_client.models.batch_response_batched_result_bulk_delete_artifact_result import BatchResponseBatchedResultBulkDeleteArtifactResult
from openapi_client.models.batch_response_batched_result_organization_invitation_base import BatchResponseBatchedResultOrganizationInvitationBase
from openapi_client.models.batched_result_bulk_delete_artifact_result import BatchedResultBulkDeleteArtifactResult
from openapi_client.models.batched_result_organization_invitation_base import BatchedResultOrganizationInvitationBase
from openapi_client.models.billing_script_response import BillingScriptResponse
from openapi_client.models.billing_version_code import BillingVersionCode
from openapi_client.models.billingscriptresponse_response import BillingscriptresponseResponse
from openapi_client.models.binding import Binding
from openapi_client.models.body_aws_marketplace_registration_api_v2_organization_billing_aws_marketplace_registration_post import BodyAwsMarketplaceRegistrationApiV2OrganizationBillingAwsMarketplaceRegistrationPost
from openapi_client.models.build import Build
from openapi_client.models.build_log_response import BuildLogResponse
from openapi_client.models.build_registration import BuildRegistration
from openapi_client.models.build_response import BuildResponse
from openapi_client.models.build_status import BuildStatus
from openapi_client.models.buildlogresponse_response import BuildlogresponseResponse
from openapi_client.models.bulk_delete_artifact_result import BulkDeleteArtifactResult
from openapi_client.models.bulk_delete_artifacts_request import BulkDeleteArtifactsRequest
from openapi_client.models.cli_usage_payload import CLIUsagePayload
from openapi_client.models.catalog_metadata import CatalogMetadata
from openapi_client.models.change_password_params import ChangePasswordParams
from openapi_client.models.cleanup_leaked_grafana_dashboard_response import CleanupLeakedGrafanaDashboardResponse
from openapi_client.models.cleanupleakedgrafanadashboardresponse_response import CleanupleakedgrafanadashboardresponseResponse
from openapi_client.models.clone_experimental_workspace import CloneExperimentalWorkspace
from openapi_client.models.cloud import Cloud
from openapi_client.models.cloud_analytics_event import CloudAnalyticsEvent
from openapi_client.models.cloud_analytics_event_cloud_provider_error import CloudAnalyticsEventCloudProviderError
from openapi_client.models.cloud_analytics_event_cloud_resource import CloudAnalyticsEventCloudResource
from openapi_client.models.cloud_analytics_event_command_name import CloudAnalyticsEventCommandName
from openapi_client.models.cloud_analytics_event_error import CloudAnalyticsEventError
from openapi_client.models.cloud_analytics_event_name import CloudAnalyticsEventName
from openapi_client.models.cloud_collaborator import CloudCollaborator
from openapi_client.models.cloud_collaborator_value import CloudCollaboratorValue
from openapi_client.models.cloud_collaborators_query import CloudCollaboratorsQuery
from openapi_client.models.cloud_config import CloudConfig
from openapi_client.models.cloud_data_bucket_access_mode import CloudDataBucketAccessMode
from openapi_client.models.cloud_data_bucket_file_type import CloudDataBucketFileType
from openapi_client.models.cloud_data_bucket_presigned_upload_info import CloudDataBucketPresignedUploadInfo
from openapi_client.models.cloud_data_bucket_presigned_upload_request import CloudDataBucketPresignedUploadRequest
from openapi_client.models.cloud_data_bucket_presigned_upload_scheme import CloudDataBucketPresignedUploadScheme
from openapi_client.models.cloud_data_bucket_presigned_url_request import CloudDataBucketPresignedUrlRequest
from openapi_client.models.cloud_data_bucket_presigned_url_response import CloudDataBucketPresignedUrlResponse
from openapi_client.models.cloud_data_bucket_presigned_url_scheme import CloudDataBucketPresignedUrlScheme
from openapi_client.models.cloud_data_bucket_request_scope import CloudDataBucketRequestScope
from openapi_client.models.cloud_deployment import CloudDeployment
from openapi_client.models.cloud_deployment_compute_config import CloudDeploymentComputeConfig
from openapi_client.models.cloud_deployment_config import CloudDeploymentConfig
from openapi_client.models.cloud_list_response import CloudListResponse
from openapi_client.models.cloud_name_options import CloudNameOptions
from openapi_client.models.cloud_overview_dashboard import CloudOverviewDashboard
from openapi_client.models.cloud_permission_removal import CloudPermissionRemoval
from openapi_client.models.cloud_permission_summary import CloudPermissionSummary
from openapi_client.models.cloud_project_collaborator import CloudProjectCollaborator
from openapi_client.models.cloud_project_collaborator_value import CloudProjectCollaboratorValue
from openapi_client.models.cloud_provider import CloudProvider
from openapi_client.models.cloud_providers import CloudProviders
from openapi_client.models.cloud_region_and_zones import CloudRegionAndZones
from openapi_client.models.cloud_region_info import CloudRegionInfo
from openapi_client.models.cloud_resource import CloudResource
from openapi_client.models.cloud_resource_additional_instance_types import CloudResourceAdditionalInstanceTypes
from openapi_client.models.cloud_resource_gcp import CloudResourceGCP
from openapi_client.models.cloud_response import CloudResponse
from openapi_client.models.cloud_state import CloudState
from openapi_client.models.cloud_status import CloudStatus
from openapi_client.models.cloud_type import CloudType
from openapi_client.models.cloud_types import CloudTypes
from openapi_client.models.cloud_version import CloudVersion
from openapi_client.models.cloud_waitlist_status import CloudWaitlistStatus
from openapi_client.models.cloud_with_cloud_resource import CloudWithCloudResource
from openapi_client.models.cloud_with_cloud_resource_gcp import CloudWithCloudResourceGCP
from openapi_client.models.cloudcollaborator_list_response import CloudcollaboratorListResponse
from openapi_client.models.clouddatabucketpresigneduploadinfo_response import ClouddatabucketpresigneduploadinfoResponse
from openapi_client.models.clouddatabucketpresignedurlresponse_response import ClouddatabucketpresignedurlresponseResponse
from openapi_client.models.clouddeployment_response import ClouddeploymentResponse
from openapi_client.models.clouddeploymentconfig_response import ClouddeploymentconfigResponse
from openapi_client.models.cloudoverviewdashboard_response import CloudoverviewdashboardResponse
from openapi_client.models.cloudregionandzones_response import CloudregionandzonesResponse
from openapi_client.models.cloudresource_response import CloudresourceResponse
from openapi_client.models.cloudresourceadditionalinstancetypes_list_response import CloudresourceadditionalinstancetypesListResponse
from openapi_client.models.cloudresourcegcp_response import CloudresourcegcpResponse
from openapi_client.models.cloudwithcloudresource_response import CloudwithcloudresourceResponse
from openapi_client.models.cloudwithcloudresourcegcp_response import CloudwithcloudresourcegcpResponse
from openapi_client.models.cluster_auth_response import ClusterAuthResponse
from openapi_client.models.cluster_dashboard_node import ClusterDashboardNode
from openapi_client.models.cluster_environments_query import ClusterEnvironmentsQuery
from openapi_client.models.cluster_event import ClusterEvent
from openapi_client.models.cluster_event_source import ClusterEventSource
from openapi_client.models.cluster_events_output import ClusterEventsOutput
from openapi_client.models.cluster_management_stack_versions import ClusterManagementStackVersions
from openapi_client.models.cluster_operation import ClusterOperation
from openapi_client.models.cluster_operation_type import ClusterOperationType
from openapi_client.models.cluster_state import ClusterState
from openapi_client.models.cluster_status import ClusterStatus
from openapi_client.models.cluster_status_details import ClusterStatusDetails
from openapi_client.models.clusterauthresponse_response import ClusterauthresponseResponse
from openapi_client.models.clusterdashboardnode_list_response import ClusterdashboardnodeListResponse
from openapi_client.models.clusterdashboardnode_response import ClusterdashboardnodeResponse
from openapi_client.models.clusterevent_list_response import ClustereventListResponse
from openapi_client.models.clustereventsoutput_response import ClustereventsoutputResponse
from openapi_client.models.clusteroperation_response import ClusteroperationResponse
from openapi_client.models.collaborator_type import CollaboratorType
from openapi_client.models.column_info import ColumnInfo
from openapi_client.models.commit_ledger_item_type import CommitLedgerItemType
from openapi_client.models.commit_ledger_record_v2 import CommitLedgerRecordV2
from openapi_client.models.complexity_level import ComplexityLevel
from openapi_client.models.compute_node_type import ComputeNodeType
from openapi_client.models.compute_stack import ComputeStack
from openapi_client.models.compute_template import ComputeTemplate
from openapi_client.models.compute_template_config import ComputeTemplateConfig
from openapi_client.models.compute_template_query import ComputeTemplateQuery
from openapi_client.models.computetemplate_response import ComputetemplateResponse
from openapi_client.models.computetemplateconfig_response import ComputetemplateconfigResponse
from openapi_client.models.connection_info import ConnectionInfo
from openapi_client.models.connection_metadata import ConnectionMetadata
from openapi_client.models.connection_type import ConnectionType
from openapi_client.models.connection_verification_result import ConnectionVerificationResult
from openapi_client.models.connectionverificationresult_response import ConnectionverificationresultResponse
from openapi_client.models.contract_info import ContractInfo
from openapi_client.models.convert_to_poc_request import ConvertToPOCRequest
from openapi_client.models.convert_to_self_service_request import ConvertToSelfServiceRequest
from openapi_client.models.create_aioa_cloud_waitlist import CreateAioaCloudWaitlist
from openapi_client.models.create_analytics_event import CreateAnalyticsEvent
from openapi_client.models.create_app_config import CreateAppConfig
from openapi_client.models.create_app_config_configuration_schema import CreateAppConfigConfigurationSchema
from openapi_client.models.create_byod_app_config import CreateBYODAppConfig
from openapi_client.models.create_byod_app_config_configuration_schema import CreateBYODAppConfigConfigurationSchema
from openapi_client.models.create_byod_build import CreateBYODBuild
from openapi_client.models.create_billing_version import CreateBillingVersion
from openapi_client.models.create_bug_report_response import CreateBugReportResponse
from openapi_client.models.create_build import CreateBuild
from openapi_client.models.create_cloud_collaborator import CreateCloudCollaborator
from openapi_client.models.create_cloud_resource import CreateCloudResource
from openapi_client.models.create_cloud_resource_gcp import CreateCloudResourceGCP
from openapi_client.models.create_cluster_compute_config import CreateClusterComputeConfig
from openapi_client.models.create_compute_template import CreateComputeTemplate
from openapi_client.models.create_compute_template_config import CreateComputeTemplateConfig
from openapi_client.models.create_dataset import CreateDataset
from openapi_client.models.create_empty_cloud import CreateEmptyCloud
from openapi_client.models.create_experimental_workspace import CreateExperimentalWorkspace
from openapi_client.models.create_instance_usage_budget import CreateInstanceUsageBudget
from openapi_client.models.create_internal_production_job import CreateInternalProductionJob
from openapi_client.models.create_job_queue_config import CreateJobQueueConfig
from openapi_client.models.create_job_queue_requests import CreateJobQueueRequests
from openapi_client.models.create_machine_pool_request import CreateMachinePoolRequest
from openapi_client.models.create_machine_pool_response import CreateMachinePoolResponse
from openapi_client.models.create_machine_request import CreateMachineRequest
from openapi_client.models.create_machine_response import CreateMachineResponse
from openapi_client.models.create_metronome_webhook_notification import CreateMetronomeWebhookNotification
from openapi_client.models.create_notification_channel_record import CreateNotificationChannelRecord
from openapi_client.models.create_otp_return_api_model import CreateOTPReturnApiModel
from openapi_client.models.create_organization_configuration import CreateOrganizationConfiguration
from openapi_client.models.create_organization_invitation import CreateOrganizationInvitation
from openapi_client.models.create_production_job_config import CreateProductionJobConfig
from openapi_client.models.create_resource_notification import CreateResourceNotification
from openapi_client.models.create_resource_quota import CreateResourceQuota
from openapi_client.models.create_schedule import CreateSchedule
from openapi_client.models.create_user import CreateUser
from openapi_client.models.create_user_project_collaborator import CreateUserProjectCollaborator
from openapi_client.models.create_user_project_collaborator_value import CreateUserProjectCollaboratorValue
from openapi_client.models.create_workspace_from_template import CreateWorkspaceFromTemplate
from openapi_client.models.create_workspace_template import CreateWorkspaceTemplate
from openapi_client.models.create_workspace_template_version import CreateWorkspaceTemplateVersion
from openapi_client.models.createbugreportresponse_response import CreatebugreportresponseResponse
from openapi_client.models.createcomputetemplateconfig_response import CreatecomputetemplateconfigResponse
from openapi_client.models.createmachinepoolresponse_response import CreatemachinepoolresponseResponse
from openapi_client.models.createmachineresponse_response import CreatemachineresponseResponse
from openapi_client.models.createotpreturnapimodel_response import CreateotpreturnapimodelResponse
from openapi_client.models.credit_grant_record_v2 import CreditGrantRecordV2
from openapi_client.models.credit_ledger_item_type import CreditLedgerItemType
from openapi_client.models.credit_ledger_record_v2 import CreditLedgerRecordV2
from openapi_client.models.credit_record_commit_v2 import CreditRecordCommitV2
from openapi_client.models.credit_record_credit_v2 import CreditRecordCreditV2
from openapi_client.models.credit_type import CreditType
from openapi_client.models.credits_v2 import CreditsV2
from openapi_client.models.customer_alert_status import CustomerAlertStatus
from openapi_client.models.dashboard_panel import DashboardPanel
from openapi_client.models.data_catalog import DataCatalog
from openapi_client.models.data_catalog_connection import DataCatalogConnection
from openapi_client.models.data_catalog_connection_status import DataCatalogConnectionStatus
from openapi_client.models.data_catalog_object_type import DataCatalogObjectType
from openapi_client.models.data_catalog_provider import DataCatalogProvider
from openapi_client.models.data_catalog_schema import DataCatalogSchema
from openapi_client.models.data_catalog_table import DataCatalogTable
from openapi_client.models.data_catalog_volume import DataCatalogVolume
from openapi_client.models.databricks_connection_config import DatabricksConnectionConfig
from openapi_client.models.databricks_connection_info import DatabricksConnectionInfo
from openapi_client.models.databricks_connection_response import DatabricksConnectionResponse
from openapi_client.models.databricks_register_request import DatabricksRegisterRequest
from openapi_client.models.databricksconnectioninfo_response import DatabricksconnectioninfoResponse
from openapi_client.models.databricksconnectionresponse_response import DatabricksconnectionresponseResponse
from openapi_client.models.datacatalog_list_response import DatacatalogListResponse
from openapi_client.models.datacatalogconnection_list_response import DatacatalogconnectionListResponse
from openapi_client.models.datacatalogschema_list_response import DatacatalogschemaListResponse
from openapi_client.models.datacatalogtable_list_response import DatacatalogtableListResponse
from openapi_client.models.datacatalogvolume_list_response import DatacatalogvolumeListResponse
from openapi_client.models.dataplane_services import DataplaneServices
from openapi_client.models.dataset import Dataset
from openapi_client.models.dataset_dag import DatasetDag
from openapi_client.models.dataset_dag_response import DatasetDagResponse
from openapi_client.models.dataset_jobs import DatasetJobs
from openapi_client.models.dataset_list_response import DatasetListResponse
from openapi_client.models.dataset_metadata import DatasetMetadata
from openapi_client.models.dataset_metadata_response import DatasetMetadataResponse
from openapi_client.models.dataset_metrics import DatasetMetrics
from openapi_client.models.dataset_metrics_response import DatasetMetricsResponse
from openapi_client.models.dataset_response import DatasetResponse
from openapi_client.models.dataset_state import DatasetState
from openapi_client.models.dataset_upload import DatasetUpload
from openapi_client.models.datasetupload_response import DatasetuploadResponse
from openapi_client.models.decorated_application_template import DecoratedApplicationTemplate
from openapi_client.models.decorated_build import DecoratedBuild
from openapi_client.models.decorated_cloud_resource import DecoratedCloudResource
from openapi_client.models.decorated_compute_template import DecoratedComputeTemplate
from openapi_client.models.decorated_compute_template_config import DecoratedComputeTemplateConfig
from openapi_client.models.decorated_interactive_session import DecoratedInteractiveSession
from openapi_client.models.decorated_job import DecoratedJob
from openapi_client.models.decorated_job_connection_info import DecoratedJobConnectionInfo
from openapi_client.models.decorated_job_queue import DecoratedJobQueue
from openapi_client.models.decorated_list_service_api_model import DecoratedListServiceAPIModel
from openapi_client.models.decorated_production_job import DecoratedProductionJob
from openapi_client.models.decorated_production_job_state_transition import DecoratedProductionJobStateTransition
from openapi_client.models.decorated_production_service_v2_api_model import DecoratedProductionServiceV2APIModel
from openapi_client.models.decorated_production_service_v2_version_api_model import DecoratedProductionServiceV2VersionAPIModel
from openapi_client.models.decorated_schedule import DecoratedSchedule
from openapi_client.models.decorated_serve_deployment import DecoratedServeDeployment
from openapi_client.models.decorated_service_event_api_model import DecoratedServiceEventAPIModel
from openapi_client.models.decorated_session import DecoratedSession
from openapi_client.models.decorated_support_request import DecoratedSupportRequest
from openapi_client.models.decorated_unified_job import DecoratedUnifiedJob
from openapi_client.models.decorated_workload_connection_info import DecoratedWorkloadConnectionInfo
from openapi_client.models.decoratedapplicationtemplate_list_response import DecoratedapplicationtemplateListResponse
from openapi_client.models.decoratedapplicationtemplate_response import DecoratedapplicationtemplateResponse
from openapi_client.models.decoratedbuild_list_response import DecoratedbuildListResponse
from openapi_client.models.decoratedbuild_response import DecoratedbuildResponse
from openapi_client.models.decoratedcloudresource_list_response import DecoratedcloudresourceListResponse
from openapi_client.models.decoratedcloudresource_response import DecoratedcloudresourceResponse
from openapi_client.models.decoratedcomputetemplate_list_response import DecoratedcomputetemplateListResponse
from openapi_client.models.decoratedcomputetemplate_response import DecoratedcomputetemplateResponse
from openapi_client.models.decoratedinteractivesession_list_response import DecoratedinteractivesessionListResponse
from openapi_client.models.decoratedinteractivesession_response import DecoratedinteractivesessionResponse
from openapi_client.models.decoratedjob_response import DecoratedjobResponse
from openapi_client.models.decoratedjobqueue_list_response import DecoratedjobqueueListResponse
from openapi_client.models.decoratedjobqueue_response import DecoratedjobqueueResponse
from openapi_client.models.decoratedlistserviceapimodel_list_response import DecoratedlistserviceapimodelListResponse
from openapi_client.models.decoratedproductionjob_list_response import DecoratedproductionjobListResponse
from openapi_client.models.decoratedproductionjob_response import DecoratedproductionjobResponse
from openapi_client.models.decoratedproductionjobstatetransition_list_response import DecoratedproductionjobstatetransitionListResponse
from openapi_client.models.decoratedproductionservicev2_apimodel_response import Decoratedproductionservicev2ApimodelResponse
from openapi_client.models.decoratedproductionservicev2_versionapimodel_list_response import Decoratedproductionservicev2VersionapimodelListResponse
from openapi_client.models.decoratedproductionservicev2_versionapimodel_response import Decoratedproductionservicev2VersionapimodelResponse
from openapi_client.models.decoratedschedule_list_response import DecoratedscheduleListResponse
from openapi_client.models.decoratedschedule_response import DecoratedscheduleResponse
from openapi_client.models.decoratedservedeployment_list_response import DecoratedservedeploymentListResponse
from openapi_client.models.decoratedservedeployment_response import DecoratedservedeploymentResponse
from openapi_client.models.decoratedserviceeventapimodel_list_response import DecoratedserviceeventapimodelListResponse
from openapi_client.models.decoratedsession_list_response import DecoratedsessionListResponse
from openapi_client.models.decoratedsession_response import DecoratedsessionResponse
from openapi_client.models.decoratedsupportrequest_list_response import DecoratedsupportrequestListResponse
from openapi_client.models.decoratedsupportrequest_response import DecoratedsupportrequestResponse
from openapi_client.models.decoratedunifiedjob_list_response import DecoratedunifiedjobListResponse
from openapi_client.models.delete_machine_pool_request import DeleteMachinePoolRequest
from openapi_client.models.delete_machine_request import DeleteMachineRequest
from openapi_client.models.delete_resource_tags_request import DeleteResourceTagsRequest
from openapi_client.models.deleted_count import DeletedCount
from openapi_client.models.deleted_platform_fine_tuned_model import DeletedPlatformFineTunedModel
from openapi_client.models.deletedcount_response import DeletedcountResponse
from openapi_client.models.deletedplatformfinetunedmodel_response import DeletedplatformfinetunedmodelResponse
from openapi_client.models.deletemachinepoolresponse_response import DeletemachinepoolresponseResponse
from openapi_client.models.deployment_infra_response import DeploymentInfraResponse
from openapi_client.models.deploymentinfraresponse_response import DeploymentinfraresponseResponse
from openapi_client.models.describe_machine_pool_machines_filters import DescribeMachinePoolMachinesFilters
from openapi_client.models.describe_machine_pool_machines_request import DescribeMachinePoolMachinesRequest
from openapi_client.models.describe_machine_pool_request import DescribeMachinePoolRequest
from openapi_client.models.describe_machine_pool_requests_filters import DescribeMachinePoolRequestsFilters
from openapi_client.models.describe_machine_pool_requests_request import DescribeMachinePoolRequestsRequest
from openapi_client.models.describe_machine_pool_response import DescribeMachinePoolResponse
from openapi_client.models.describe_machine_pool_workloads_filters import DescribeMachinePoolWorkloadsFilters
from openapi_client.models.describe_machine_pool_workloads_request import DescribeMachinePoolWorkloadsRequest
from openapi_client.models.describe_system_workload_response import DescribeSystemWorkloadResponse
from openapi_client.models.describemachinepoolresponse_response import DescribemachinepoolresponseResponse
from openapi_client.models.describesystemworkloadresponse_response import DescribesystemworkloadresponseResponse
from openapi_client.models.detach_machine_pool_from_cloud_request import DetachMachinePoolFromCloudRequest
from openapi_client.models.detachmachinepoolfromcloudresponse_response import DetachmachinepoolfromcloudresponseResponse
from openapi_client.models.directory_sync import DirectorySync
from openapi_client.models.directory_sync_readiness import DirectorySyncReadiness
from openapi_client.models.directorysyncreadiness_response import DirectorysyncreadinessResponse
from openapi_client.models.domain_verification import DomainVerification
from openapi_client.models.dry_run_permission_diff_result import DryRunPermissionDiffResult
from openapi_client.models.dryrunpermissiondiffresult_response import DryrunpermissiondiffresultResponse
from openapi_client.models.editable_cloud_resource import EditableCloudResource
from openapi_client.models.editable_cloud_resource_gcp import EditableCloudResourceGCP
from openapi_client.models.email_verification_request import EmailVerificationRequest
from openapi_client.models.email_verification_response import EmailVerificationResponse
from openapi_client.models.emailverificationresponse_response import EmailverificationresponseResponse
from openapi_client.models.entity_type import EntityType
from openapi_client.models.error import Error
from openapi_client.models.event_level import EventLevel
from openapi_client.models.execute_command_response import ExecuteCommandResponse
from openapi_client.models.execute_interactive_command_options import ExecuteInteractiveCommandOptions
from openapi_client.models.executecommandresponse_response import ExecutecommandresponseResponse
from openapi_client.models.experimental_workspace import ExperimentalWorkspace
from openapi_client.models.experimental_workspaces_sort_field import ExperimentalWorkspacesSortField
from openapi_client.models.experimentalworkspace_list_response import ExperimentalworkspaceListResponse
from openapi_client.models.experimentalworkspace_response import ExperimentalworkspaceResponse
from openapi_client.models.extend_poc_request import ExtendPOCRequest
from openapi_client.models.external_terminal_command import ExternalTerminalCommand
from openapi_client.models.feature_flag_response import FeatureFlagResponse
from openapi_client.models.featureflagresponse_response import FeatureflagresponseResponse
from openapi_client.models.file_credential import FileCredential
from openapi_client.models.file_storage import FileStorage
from openapi_client.models.fine_tune_type import FineTuneType
from openapi_client.models.fine_tuned_model import FineTunedModel
from openapi_client.models.finetunedmodel_list_response import FinetunedmodelListResponse
from openapi_client.models.finetunedmodel_response import FinetunedmodelResponse
from openapi_client.models.gcp_config import GCPConfig
from openapi_client.models.gcp_file_store_config import GCPFileStoreConfig
from openapi_client.models.gcp_memorystore_instance_config import GCPMemorystoreInstanceConfig
from openapi_client.models.gpu_usage import GPUUsage
from openapi_client.models.get_or_create_build_from_image_uri_request import GetOrCreateBuildFromImageUriRequest
from openapi_client.models.global_workspace_template import GlobalWorkspaceTemplate
from openapi_client.models.globalworkspacetemplate_list_response import GlobalworkspacetemplateListResponse
from openapi_client.models.grafana_dashboard import GrafanaDashboard
from openapi_client.models.grpc_protocol_config import GrpcProtocolConfig
from openapi_client.models.http_validation_error import HTTPValidationError
from openapi_client.models.ha_job_error_types import HaJobErrorTypes
from openapi_client.models.ha_job_event_origin import HaJobEventOrigin
from openapi_client.models.ha_job_goal_states import HaJobGoalStates
from openapi_client.models.ha_job_states import HaJobStates
from openapi_client.models.ha_job_type import HaJobType
from openapi_client.models.ha_jobs_sort_field import HaJobsSortField
from openapi_client.models.head_ip import HeadIp
from openapi_client.models.headip_response import HeadipResponse
from openapi_client.models.http_protocol_config import HttpProtocolConfig
from openapi_client.models.i_know_response import IKnowResponse
from openapi_client.models.i_know_time_series_event import IKnowTimeSeriesEvent
from openapi_client.models.idle_termination_status import IdleTerminationStatus
from openapi_client.models.import_cluster_environment import ImportClusterEnvironment
from openapi_client.models.instance_usage_budget import InstanceUsageBudget
from openapi_client.models.instance_usage_budget_evaluation_period import InstanceUsageBudgetEvaluationPeriod
from openapi_client.models.instance_usage_budget_unit import InstanceUsageBudgetUnit
from openapi_client.models.instanceusagebudget_list_response import InstanceusagebudgetListResponse
from openapi_client.models.instanceusagebudget_response import InstanceusagebudgetResponse
from openapi_client.models.integration_details import IntegrationDetails
from openapi_client.models.internal_production_job import InternalProductionJob
from openapi_client.models.internalproductionjob_response import InternalproductionjobResponse
from openapi_client.models.job_access import JobAccess
from openapi_client.models.job_details import JobDetails
from openapi_client.models.job_event_fields import JobEventFields
from openapi_client.models.job_queue import JobQueue
from openapi_client.models.job_queue_config import JobQueueConfig
from openapi_client.models.job_queue_execution_mode import JobQueueExecutionMode
from openapi_client.models.job_queue_sort_directive import JobQueueSortDirective
from openapi_client.models.job_queue_sort_field import JobQueueSortField
from openapi_client.models.job_queue_spec import JobQueueSpec
from openapi_client.models.job_queue_state import JobQueueState
from openapi_client.models.job_queue_termination_response import JobQueueTerminationResponse
from openapi_client.models.job_queues_query import JobQueuesQuery
from openapi_client.models.job_report import JobReport
from openapi_client.models.job_run_type import JobRunType
from openapi_client.models.job_state_log_level_types import JobStateLogLevelTypes
from openapi_client.models.job_with_report import JobWithReport
from openapi_client.models.jobqueue_response import JobqueueResponse
from openapi_client.models.jobqueueterminationresponse_response import JobqueueterminationresponseResponse
from openapi_client.models.jobs_sort_field import JobsSortField
from openapi_client.models.jobwithreport_list_response import JobwithreportListResponse
from openapi_client.models.json_patch_operation import JsonPatchOperation
from openapi_client.models.kubernetes_config import KubernetesConfig
from openapi_client.models.kubernetes_manager_registration_request import KubernetesManagerRegistrationRequest
from openapi_client.models.kubernetes_manager_registration_response import KubernetesManagerRegistrationResponse
from openapi_client.models.kubernetesmanagerregistrationresponse_response import KubernetesmanagerregistrationresponseResponse
from openapi_client.models.lb_resource import LBResource
from openapi_client.models.lbresource_response import LbresourceResponse
from openapi_client.models.lineage_artifact import LineageArtifact
from openapi_client.models.lineage_artifact_sort_field import LineageArtifactSortField
from openapi_client.models.lineage_artifact_type import LineageArtifactType
from openapi_client.models.lineage_direction import LineageDirection
from openapi_client.models.lineage_graph import LineageGraph
from openapi_client.models.lineage_graph_node import LineageGraphNode
from openapi_client.models.lineage_node_type import LineageNodeType
from openapi_client.models.lineage_workload import LineageWorkload
from openapi_client.models.lineage_workload_sort_field import LineageWorkloadSortField
from openapi_client.models.lineage_workload_type import LineageWorkloadType
from openapi_client.models.lineageartifact_list_response import LineageartifactListResponse
from openapi_client.models.lineageartifact_response import LineageartifactResponse
from openapi_client.models.lineagegraph_response import LineagegraphResponse
from openapi_client.models.lineageworkload_list_response import LineageworkloadListResponse
from openapi_client.models.lineageworkload_response import LineageworkloadResponse
from openapi_client.models.list_databricks_connections import ListDatabricksConnections
from openapi_client.models.list_machine_pools_response import ListMachinePoolsResponse
from openapi_client.models.list_machines_response import ListMachinesResponse
from openapi_client.models.list_ray_sessions_response import ListRaySessionsResponse
from openapi_client.models.list_resource_notifications_query import ListResourceNotificationsQuery
from openapi_client.models.list_resource_quotas_query import ListResourceQuotasQuery
from openapi_client.models.list_response_metadata import ListResponseMetadata
from openapi_client.models.list_user_group_memberships_result import ListUserGroupMembershipsResult
from openapi_client.models.listmachinepoolsresponse_response import ListmachinepoolsresponseResponse
from openapi_client.models.listmachinesresponse_response import ListmachinesresponseResponse
from openapi_client.models.listraysessionsresponse_response import ListraysessionsresponseResponse
from openapi_client.models.listusergroupmembershipsresult_response import ListusergroupmembershipsresultResponse
from openapi_client.models.log_download_config import LogDownloadConfig
from openapi_client.models.log_download_request import LogDownloadRequest
from openapi_client.models.log_download_result import LogDownloadResult
from openapi_client.models.log_file_chunk import LogFileChunk
from openapi_client.models.log_filter import LogFilter
from openapi_client.models.log_item import LogItem
from openapi_client.models.log_item_batch import LogItemBatch
from openapi_client.models.logdownloadresult_response import LogdownloadresultResponse
from openapi_client.models.login_user_params import LoginUserParams
from openapi_client.models.logitembatch_response import LogitembatchResponse
from openapi_client.models.logs_output import LogsOutput
from openapi_client.models.logsoutput_response import LogsoutputResponse
from openapi_client.models.long_running_workload import LongRunningWorkload
from openapi_client.models.longrunningworkload_list_response import LongrunningworkloadListResponse
from openapi_client.models.machine_allocation_state import MachineAllocationState
from openapi_client.models.machine_connection_state import MachineConnectionState
from openapi_client.models.machine_info import MachineInfo
from openapi_client.models.machine_launch_failure import MachineLaunchFailure
from openapi_client.models.machine_pool import MachinePool
from openapi_client.models.machine_pool_search_query import MachinePoolSearchQuery
from openapi_client.models.machine_pool_search_result import MachinePoolSearchResult
from openapi_client.models.machine_state_info import MachineStateInfo
from openapi_client.models.machine_type_partition_filter import MachineTypePartitionFilter
from openapi_client.models.machinepoolsearchresult_list_response import MachinepoolsearchresultListResponse
from openapi_client.models.machinestateinfo_list_response import MachinestateinfoListResponse
from openapi_client.models.metric import Metric
from openapi_client.models.metrics_query_response import MetricsQueryResponse
from openapi_client.models.metricsqueryresponse_response import MetricsqueryresponseResponse
from openapi_client.models.metronome_customer_info_model import MetronomeCustomerInfoModel
from openapi_client.models.metronome_dashboard_type import MetronomeDashboardType
from openapi_client.models.metronomecustomerinfomodel_list_response import MetronomecustomerinfomodelListResponse
from openapi_client.models.metronomecustomerinfomodel_response import MetronomecustomerinfomodelResponse
from openapi_client.models.mini_build import MiniBuild
from openapi_client.models.mini_cloud import MiniCloud
from openapi_client.models.mini_cluster import MiniCluster
from openapi_client.models.mini_compute_template import MiniComputeTemplate
from openapi_client.models.mini_compute_template_config import MiniComputeTemplateConfig
from openapi_client.models.mini_job_run import MiniJobRun
from openapi_client.models.mini_namespace import MiniNamespace
from openapi_client.models.mini_organization import MiniOrganization
from openapi_client.models.mini_production_job import MiniProductionJob
from openapi_client.models.mini_project import MiniProject
from openapi_client.models.mini_runtime_environment import MiniRuntimeEnvironment
from openapi_client.models.mini_schedule import MiniSchedule
from openapi_client.models.mini_user import MiniUser
from openapi_client.models.minibuild_list_response import MinibuildListResponse
from openapi_client.models.minicomputetemplate_list_response import MinicomputetemplateListResponse
from openapi_client.models.nfs_mount_target import NFSMountTarget
from openapi_client.models.networking_mode import NetworkingMode
from openapi_client.models.node_metrics import NodeMetrics
from openapi_client.models.node_metrics_response import NodeMetricsResponse
from openapi_client.models.node_registration_aws import NodeRegistrationAWS
from openapi_client.models.node_registration_gcp import NodeRegistrationGCP
from openapi_client.models.node_registration_k8_s import NodeRegistrationK8S
from openapi_client.models.node_registration_provisioned import NodeRegistrationProvisioned
from openapi_client.models.node_registration_v2 import NodeRegistrationV2
from openapi_client.models.node_status import NodeStatus
from openapi_client.models.node_type import NodeType
from openapi_client.models.nodemetricsresponse_response import NodemetricsresponseResponse
from openapi_client.models.notification_channel_email_config import NotificationChannelEmailConfig
from openapi_client.models.notification_channel_slack_config import NotificationChannelSlackConfig
from openapi_client.models.notification_channel_webhook_config import NotificationChannelWebhookConfig
from openapi_client.models.oauth_auth_url_response import OauthAuthUrlResponse
from openapi_client.models.object_storage import ObjectStorage
from openapi_client.models.operation_error import OperationError
from openapi_client.models.operation_progress import OperationProgress
from openapi_client.models.operation_result import OperationResult
from openapi_client.models.operator_branch import OperatorBranch
from openapi_client.models.operator_check_result import OperatorCheckResult
from openapi_client.models.operator_check_status import OperatorCheckStatus
from openapi_client.models.operator_event import OperatorEvent
from openapi_client.models.operator_event_response import OperatorEventResponse
from openapi_client.models.operator_event_type import OperatorEventType
from openapi_client.models.operator_id import OperatorId
from openapi_client.models.operator_metadata import OperatorMetadata
from openapi_client.models.operator_metrics import OperatorMetrics
from openapi_client.models.operator_schema_response import OperatorSchemaResponse
from openapi_client.models.operator_status import OperatorStatus
from openapi_client.models.operator_status_details import OperatorStatusDetails
from openapi_client.models.org_owner import OrgOwner
from openapi_client.models.org_permission_change import OrgPermissionChange
from openapi_client.models.organization import Organization
from openapi_client.models.organization_additional_role import OrganizationAdditionalRole
from openapi_client.models.organization_base_role import OrganizationBaseRole
from openapi_client.models.organization_collaborator import OrganizationCollaborator
from openapi_client.models.organization_collaborators_query import OrganizationCollaboratorsQuery
from openapi_client.models.organization_configuration import OrganizationConfiguration
from openapi_client.models.organization_configuration_response import OrganizationConfigurationResponse
from openapi_client.models.organization_contract_info import OrganizationContractInfo
from openapi_client.models.organization_invitation import OrganizationInvitation
from openapi_client.models.organization_invitation_base import OrganizationInvitationBase
from openapi_client.models.organization_marketing_questions import OrganizationMarketingQuestions
from openapi_client.models.organization_permission_level import OrganizationPermissionLevel
from openapi_client.models.organization_summary import OrganizationSummary
from openapi_client.models.organization_usage_alert import OrganizationUsageAlert
from openapi_client.models.organization_usage_alert_severity import OrganizationUsageAlertSeverity
from openapi_client.models.organization_user_groups_collaborator import OrganizationUserGroupsCollaborator
from openapi_client.models.organization_user_groups_collaborators_query import OrganizationUserGroupsCollaboratorsQuery
from openapi_client.models.organizationcollaborator_list_response import OrganizationcollaboratorListResponse
from openapi_client.models.organizationcollaborator_response import OrganizationcollaboratorResponse
from openapi_client.models.organizationconfiguration_list_response import OrganizationconfigurationListResponse
from openapi_client.models.organizationconfigurationresponse_response import OrganizationconfigurationresponseResponse
from openapi_client.models.organizationcontractinfo_response import OrganizationcontractinfoResponse
from openapi_client.models.organizationinvitation_list_response import OrganizationinvitationListResponse
from openapi_client.models.organizationinvitation_response import OrganizationinvitationResponse
from openapi_client.models.organizationinvitationbase_response import OrganizationinvitationbaseResponse
from openapi_client.models.organizationusagealert_list_response import OrganizationusagealertListResponse
from openapi_client.models.organizationusergroupscollaborator_list_response import OrganizationusergroupscollaboratorListResponse
from openapi_client.models.page_query import PageQuery
from openapi_client.models.partition_info import PartitionInfo
from openapi_client.models.pause_schedule import PauseSchedule
from openapi_client.models.permission_check_request import PermissionCheckRequest
from openapi_client.models.permission_check_response import PermissionCheckResponse
from openapi_client.models.permission_level import PermissionLevel
from openapi_client.models.permission_type import PermissionType
from openapi_client.models.permissioncheckresponse_response import PermissioncheckresponseResponse
from openapi_client.models.physical_resources import PhysicalResources
from openapi_client.models.plan_status import PlanStatus
from openapi_client.models.policy_response import PolicyResponse
from openapi_client.models.policy_sync_status import PolicySyncStatus
from openapi_client.models.policyresponse_response import PolicyresponseResponse
from openapi_client.models.presigned_url_response import PresignedUrlResponse
from openapi_client.models.production_job import ProductionJob
from openapi_client.models.production_job_config import ProductionJobConfig
from openapi_client.models.production_job_event import ProductionJobEvent
from openapi_client.models.production_job_event_scope_filter import ProductionJobEventScopeFilter
from openapi_client.models.production_job_state_transition import ProductionJobStateTransition
from openapi_client.models.productionjob_response import ProductionjobResponse
from openapi_client.models.productionjobevent_list_response import ProductionjobeventListResponse
from openapi_client.models.project import Project
from openapi_client.models.project_base import ProjectBase
from openapi_client.models.project_collaborator import ProjectCollaborator
from openapi_client.models.project_collaborator_value import ProjectCollaboratorValue
from openapi_client.models.project_default_session_name import ProjectDefaultSessionName
from openapi_client.models.project_list_response import ProjectListResponse
from openapi_client.models.project_permission_removal import ProjectPermissionRemoval
from openapi_client.models.project_permission_summary import ProjectPermissionSummary
from openapi_client.models.project_response import ProjectResponse
from openapi_client.models.projectbase_response import ProjectbaseResponse
from openapi_client.models.projectcollaborator_list_response import ProjectcollaboratorListResponse
from openapi_client.models.projectdefaultsessionname_response import ProjectdefaultsessionnameResponse
from openapi_client.models.projects_sort_field import ProjectsSortField
from openapi_client.models.protocols import Protocols
from openapi_client.models.published_filter import PublishedFilter
from openapi_client.models.python_modules import PythonModules
from openapi_client.models.quota import Quota
from openapi_client.models.ray_gcs_external_storage_config import RayGCSExternalStorageConfig
from openapi_client.models.ray_runtime_env_config import RayRuntimeEnvConfig
from openapi_client.models.ray_session import RaySession
from openapi_client.models.read_billing_version import ReadBillingVersion
from openapi_client.models.readbillingversion_list_response import ReadbillingversionListResponse
from openapi_client.models.redaction_request import RedactionRequest
from openapi_client.models.redaction_result import RedactionResult
from openapi_client.models.replica_details import ReplicaDetails
from openapi_client.models.replica_state import ReplicaState
from openapi_client.models.request_email_magic_link_response import RequestEmailMagicLinkResponse
from openapi_client.models.request_otp_return_api_model import RequestOTPReturnApiModel
from openapi_client.models.request_password_reset_params import RequestPasswordResetParams
from openapi_client.models.request_state_info import RequestStateInfo
from openapi_client.models.requestemailmagiclinkresponse_response import RequestemailmagiclinkresponseResponse
from openapi_client.models.requestotpreturnapimodel_response import RequestotpreturnapimodelResponse
from openapi_client.models.requeststateinfo_list_response import RequeststateinfoListResponse
from openapi_client.models.reset_password_params import ResetPasswordParams
from openapi_client.models.resource_alert_event_type import ResourceAlertEventType
from openapi_client.models.resource_notification import ResourceNotification
from openapi_client.models.resource_permission_change import ResourcePermissionChange
from openapi_client.models.resource_policy_item import ResourcePolicyItem
from openapi_client.models.resource_quota import ResourceQuota
from openapi_client.models.resource_quota_status import ResourceQuotaStatus
from openapi_client.models.resource_tag_record import ResourceTagRecord
from openapi_client.models.resource_tag_resource_type import ResourceTagResourceType
from openapi_client.models.resource_tags_list import ResourceTagsList
from openapi_client.models.resource_type import ResourceType
from openapi_client.models.resourcenotification_list_response import ResourcenotificationListResponse
from openapi_client.models.resourcenotification_response import ResourcenotificationResponse
from openapi_client.models.resourcepolicyitem_list_response import ResourcepolicyitemListResponse
from openapi_client.models.resourcequota_list_response import ResourcequotaListResponse
from openapi_client.models.resourcequota_response import ResourcequotaResponse
from openapi_client.models.resources import Resources
from openapi_client.models.resourcetagslist_response import ResourcetagslistResponse
from openapi_client.models.revoke_api_keys_request import RevokeApiKeysRequest
from openapi_client.models.revoke_api_keys_response import RevokeApiKeysResponse
from openapi_client.models.revokeapikeysresponse_response import RevokeapikeysresponseResponse
from openapi_client.models.rollback_service_model import RollbackServiceModel
from openapi_client.models.rollout_strategy import RolloutStrategy
from openapi_client.models.run_attempt_status import RunAttemptStatus
from openapi_client.models.run_status import RunStatus
from openapi_client.models.s3_download_location import S3DownloadLocation
from openapi_client.models.scim_migration_result import SCIMMigrationResult
from openapi_client.models.sso_config import SSOConfig
from openapi_client.models.sso_connection import SSOConnection
from openapi_client.models.sso_connection_state import SSOConnectionState
from openapi_client.models.sso_login_info import SSOLoginInfo
from openapi_client.models.sso_mode import SSOMode
from openapi_client.models.supportedbaseimagesenum import SUPPORTEDBASEIMAGESENUM
from openapi_client.models.schedule_config import ScheduleConfig
from openapi_client.models.scheduler_info import SchedulerInfo
from openapi_client.models.schema_metadata import SchemaMetadata
from openapi_client.models.scim_list_user_permissions_result import ScimListUserPermissionsResult
from openapi_client.models.scim_user_facing_role import ScimUserFacingRole
from openapi_client.models.scimlistuserpermissionsresult_response import ScimlistuserpermissionsresultResponse
from openapi_client.models.scimmigrationresult_response import ScimmigrationresultResponse
from openapi_client.models.serve_deployment_grafana_dashboard_status import ServeDeploymentGrafanaDashboardStatus
from openapi_client.models.serve_deployment_state import ServeDeploymentState
from openapi_client.models.server_session_token import ServerSessionToken
from openapi_client.models.serversessiontoken_response import ServersessiontokenResponse
from openapi_client.models.service_config import ServiceConfig
from openapi_client.models.service_event_current_state import ServiceEventCurrentState
from openapi_client.models.service_event_fields import ServiceEventFields
from openapi_client.models.service_event_origin import ServiceEventOrigin
from openapi_client.models.service_event_scope import ServiceEventScope
from openapi_client.models.service_event_scope_filter import ServiceEventScopeFilter
from openapi_client.models.service_event_type import ServiceEventType
from openapi_client.models.service_event_verbose_message_model import ServiceEventVerboseMessageModel
from openapi_client.models.service_goal_states import ServiceGoalStates
from openapi_client.models.service_observability_urls import ServiceObservabilityUrls
from openapi_client.models.service_sort_field import ServiceSortField
from openapi_client.models.service_type import ServiceType
from openapi_client.models.service_version_sort_field import ServiceVersionSortField
from openapi_client.models.service_version_state import ServiceVersionState
from openapi_client.models.serviceeventverbosemessagemodel_list_response import ServiceeventverbosemessagemodelListResponse
from openapi_client.models.session import Session
from openapi_client.models.session_access import SessionAccess
from openapi_client.models.session_command import SessionCommand
from openapi_client.models.session_command_finish_options import SessionCommandFinishOptions
from openapi_client.models.session_command_id import SessionCommandId
from openapi_client.models.session_command_types import SessionCommandTypes
from openapi_client.models.session_list_response import SessionListResponse
from openapi_client.models.session_response import SessionResponse
from openapi_client.models.session_ssh_key import SessionSshKey
from openapi_client.models.session_starting_up_data import SessionStartingUpData
from openapi_client.models.session_state import SessionState
from openapi_client.models.session_state_data import SessionStateData
from openapi_client.models.session_stopping_data import SessionStoppingData
from openapi_client.models.sessioncommand_list_response import SessioncommandListResponse
from openapi_client.models.sessioncommandid_response import SessioncommandidResponse
from openapi_client.models.sessions_sort_field import SessionsSortField
from openapi_client.models.sessionsshkey_response import SessionsshkeyResponse
from openapi_client.models.set_organization_roles import SetOrganizationRoles
from openapi_client.models.show_otp_source_return_api_model import ShowOTPSourceReturnApiModel
from openapi_client.models.showotpsourcereturnapimodel_response import ShowotpsourcereturnapimodelResponse
from openapi_client.models.sort_by_clause import SortByClause
from openapi_client.models.sort_order import SortOrder
from openapi_client.models.ssoconfig_response import SsoconfigResponse
from openapi_client.models.ssologininfo_response import SsologininfoResponse
from openapi_client.models.start_session_options import StartSessionOptions
from openapi_client.models.stop_session_options import StopSessionOptions
from openapi_client.models.stream_publish_request import StreamPublishRequest
from openapi_client.models.subnet_id_with_availability_zone_aws import SubnetIdWithAvailabilityZoneAWS
from openapi_client.models.summarize_machine_pool_request import SummarizeMachinePoolRequest
from openapi_client.models.summarize_machine_pool_response import SummarizeMachinePoolResponse
from openapi_client.models.summarizemachinepoolresponse_response import SummarizemachinepoolresponseResponse
from openapi_client.models.support_requests_query import SupportRequestsQuery
from openapi_client.models.supported_base_image_metadata import SupportedBaseImageMetadata
from openapi_client.models.supported_base_images_response import SupportedBaseImagesResponse
from openapi_client.models.supportedbaseimagesresponse_response import SupportedbaseimagesresponseResponse
from openapi_client.models.system_workload_name import SystemWorkloadName
from openapi_client.models.table_data_preview import TableDataPreview
from openapi_client.models.table_redaction_result import TableRedactionResult
from openapi_client.models.tag_key import TagKey
from openapi_client.models.tag_key_value import TagKeyValue
from openapi_client.models.tag_search_in import TagSearchIn
from openapi_client.models.tag_value import TagValue
from openapi_client.models.tagkey_list_response import TagkeyListResponse
from openapi_client.models.tagkeyvalue_list_response import TagkeyvalueListResponse
from openapi_client.models.tagvalue_list_response import TagvalueListResponse
from openapi_client.models.task_attempts import TaskAttempts
from openapi_client.models.task_exception_group_aggregate import TaskExceptionGroupAggregate
from openapi_client.models.task_exception_group_aggregate_response import TaskExceptionGroupAggregateResponse
from openapi_client.models.task_function_name_group_aggregate import TaskFunctionNameGroupAggregate
from openapi_client.models.task_function_name_group_aggregate_response import TaskFunctionNameGroupAggregateResponse
from openapi_client.models.task_grouped_aggregate_metrics import TaskGroupedAggregateMetrics
from openapi_client.models.task_job_group_aggregate import TaskJobGroupAggregate
from openapi_client.models.task_job_group_aggregate_response import TaskJobGroupAggregateResponse
from openapi_client.models.task_state import TaskState
from openapi_client.models.task_summary import TaskSummary
from openapi_client.models.task_summary_config import TaskSummaryConfig
from openapi_client.models.task_table_config import TaskTableConfig
from openapi_client.models.task_table_response import TaskTableResponse
from openapi_client.models.task_table_row import TaskTableRow
from openapi_client.models.task_type import TaskType
from openapi_client.models.tasksummary_response import TasksummaryResponse
from openapi_client.models.text_query import TextQuery
from openapi_client.models.tool import Tool
from openapi_client.models.tracing_config import TracingConfig
from openapi_client.models.train_resources import TrainResources
from openapi_client.models.train_run import TrainRun
from openapi_client.models.train_run_attempt import TrainRunAttempt
from openapi_client.models.train_worker import TrainWorker
from openapi_client.models.trainrun_list_response import TrainrunListResponse
from openapi_client.models.try_login_email_response import TryLoginEmailResponse
from openapi_client.models.try_login_work_os_response import TryLoginWorkOSResponse
from openapi_client.models.tryloginemailresponse_response import TryloginemailresponseResponse
from openapi_client.models.tryloginworkosresponse_response import TryloginworkosresponseResponse
from openapi_client.models.ux_instance import UXInstance
from openapi_client.models.unified_event import UnifiedEvent
from openapi_client.models.unified_job_sort_field import UnifiedJobSortField
from openapi_client.models.unified_job_status import UnifiedJobStatus
from openapi_client.models.unified_job_type import UnifiedJobType
from openapi_client.models.unified_origin_filter import UnifiedOriginFilter
from openapi_client.models.unifiedevent_list_response import UnifiedeventListResponse
from openapi_client.models.update_cloud_collaborator import UpdateCloudCollaborator
from openapi_client.models.update_cloud_with_cloud_resource import UpdateCloudWithCloudResource
from openapi_client.models.update_cloud_with_cloud_resource_gcp import UpdateCloudWithCloudResourceGCP
from openapi_client.models.update_cluster_dns import UpdateClusterDns
from openapi_client.models.update_job_queue_request import UpdateJobQueueRequest
from openapi_client.models.update_machine_pool_request import UpdateMachinePoolRequest
from openapi_client.models.update_organization_collaborator import UpdateOrganizationCollaborator
from openapi_client.models.update_payment_info import UpdatePaymentInfo
from openapi_client.models.update_policy_request import UpdatePolicyRequest
from openapi_client.models.update_project_collaborator import UpdateProjectCollaborator
from openapi_client.models.update_resource_quota import UpdateResourceQuota
from openapi_client.models.update_workspace_template import UpdateWorkspaceTemplate
from openapi_client.models.updatemachinepoolresponse_response import UpdatemachinepoolresponseResponse
from openapi_client.models.upload_session_command_logs_locations import UploadSessionCommandLogsLocations
from openapi_client.models.uploadsessioncommandlogslocations_response import UploadsessioncommandlogslocationsResponse
from openapi_client.models.upsert_resource_tags_request import UpsertResourceTagsRequest
from openapi_client.models.usage_by_cloud import UsageByCloud
from openapi_client.models.usage_by_cluster import UsageByCluster
from openapi_client.models.usage_by_cluster_type import UsageByClusterType
from openapi_client.models.usage_by_instance_type import UsageByInstanceType
from openapi_client.models.usage_by_project import UsageByProject
from openapi_client.models.usage_by_user import UsageByUser
from openapi_client.models.usagebycloud_list_response import UsagebycloudListResponse
from openapi_client.models.usagebycluster_list_response import UsagebyclusterListResponse
from openapi_client.models.usagebyclustertype_list_response import UsagebyclustertypeListResponse
from openapi_client.models.usagebyinstancetype_list_response import UsagebyinstancetypeListResponse
from openapi_client.models.usagebyproject_list_response import UsagebyprojectListResponse
from openapi_client.models.usagebyuser_list_response import UsagebyuserListResponse
from openapi_client.models.use_work_os_response import UseWorkOSResponse
from openapi_client.models.user_cloud_permissions import UserCloudPermissions
from openapi_client.models.user_group import UserGroup
from openapi_client.models.user_group_member import UserGroupMember
from openapi_client.models.user_group_with_members import UserGroupWithMembers
from openapi_client.models.user_info import UserInfo
from openapi_client.models.user_migration_result import UserMigrationResult
from openapi_client.models.user_permission_diff import UserPermissionDiff
from openapi_client.models.user_resend_email_options import UserResendEmailOptions
from openapi_client.models.user_service_access_types import UserServiceAccessTypes
from openapi_client.models.user_to_remove import UserToRemove
from openapi_client.models.usergroup_list_response import UsergroupListResponse
from openapi_client.models.usergroup_response import UsergroupResponse
from openapi_client.models.userinfo_response import UserinfoResponse
from openapi_client.models.useworkosresponse_response import UseworkosresponseResponse
from openapi_client.models.utm_fields import UtmFields
from openapi_client.models.validate_otp_params_api_model import ValidateOTPParamsApiModel
from openapi_client.models.validation_error import ValidationError
from openapi_client.models.validation_status import ValidationStatus
from openapi_client.models.volume_metadata import VolumeMetadata
from openapi_client.models.waitlist_status_response import WaitlistStatusResponse
from openapi_client.models.waitlist_status_type import WaitlistStatusType
from openapi_client.models.waitliststatusresponse_response import WaitliststatusresponseResponse
from openapi_client.models.wand_b_run_details import WandBRunDetails
from openapi_client.models.work_os_authorize_url_response import WorkOSAuthorizeUrlResponse
from openapi_client.models.work_os_event_status import WorkOSEventStatus
from openapi_client.models.worker_node_type import WorkerNodeType
from openapi_client.models.workload_connection_credentials import WorkloadConnectionCredentials
from openapi_client.models.workload_info import WorkloadInfo
from openapi_client.models.workload_machine_info import WorkloadMachineInfo
from openapi_client.models.workload_state_info import WorkloadStateInfo
from openapi_client.models.workload_type import WorkloadType
from openapi_client.models.workloadstateinfo_list_response import WorkloadstateinfoListResponse
from openapi_client.models.workosauthorizeurlresponse_response import WorkosauthorizeurlresponseResponse
from openapi_client.models.workspace_connection_info import WorkspaceConnectionInfo
from openapi_client.models.workspace_dataplane_artifact import WorkspaceDataplaneArtifact
from openapi_client.models.workspace_dataplane_artifacts import WorkspaceDataplaneArtifacts
from openapi_client.models.workspace_dataplane_proxied_artifacts import WorkspaceDataplaneProxiedArtifacts
from openapi_client.models.workspace_event import WorkspaceEvent
from openapi_client.models.workspace_event_fields import WorkspaceEventFields
from openapi_client.models.workspace_event_source import WorkspaceEventSource
from openapi_client.models.workspace_event_source_filter import WorkspaceEventSourceFilter
from openapi_client.models.workspace_readme import WorkspaceReadme
from openapi_client.models.workspace_snapshot_states import WorkspaceSnapshotStates
from openapi_client.models.workspace_system_artifacts import WorkspaceSystemArtifacts
from openapi_client.models.workspace_template import WorkspaceTemplate
from openapi_client.models.workspace_template_category import WorkspaceTemplateCategory
from openapi_client.models.workspace_template_cluster_environment_metadata import WorkspaceTemplateClusterEnvironmentMetadata
from openapi_client.models.workspace_template_complexity import WorkspaceTemplateComplexity
from openapi_client.models.workspace_template_icon_background_color import WorkspaceTemplateIconBackgroundColor
from openapi_client.models.workspace_template_icon_type import WorkspaceTemplateIconType
from openapi_client.models.workspace_template_readme import WorkspaceTemplateReadme
from openapi_client.models.workspace_template_version import WorkspaceTemplateVersion
from openapi_client.models.workspace_template_version_data_object import WorkspaceTemplateVersionDataObject
from openapi_client.models.workspace_templates_sort_field import WorkspaceTemplatesSortField
from openapi_client.models.workspaceconnectioninfo_list_response import WorkspaceconnectioninfoListResponse
from openapi_client.models.workspaceconnectioninfo_response import WorkspaceconnectioninfoResponse
from openapi_client.models.workspacedataplaneartifacts_response import WorkspacedataplaneartifactsResponse
from openapi_client.models.workspacedataplaneproxiedartifacts_response import WorkspacedataplaneproxiedartifactsResponse
from openapi_client.models.workspaceevent_list_response import WorkspaceeventListResponse
from openapi_client.models.workspacereadme_response import WorkspacereadmeResponse
from openapi_client.models.workspacetemplate_list_response import WorkspacetemplateListResponse
from openapi_client.models.workspacetemplate_response import WorkspacetemplateResponse
from openapi_client.models.workspacetemplateclusterenvironmentmetadata_response import WorkspacetemplateclusterenvironmentmetadataResponse
from openapi_client.models.workspacetemplatereadme_response import WorkspacetemplatereadmeResponse
from openapi_client.models.workspacetemplateversion_list_response import WorkspacetemplateversionListResponse
from openapi_client.models.workspacetemplateversion_response import WorkspacetemplateversionResponse
from openapi_client.models.write_cloud import WriteCloud
from openapi_client.models.write_cluster_config import WriteClusterConfig
from openapi_client.models.write_project import WriteProject
from openapi_client.models.write_support_request import WriteSupportRequest



import warnings
from abc import ABCMeta
from copy import deepcopy
from enum import Enum
from functools import partial
from inspect import Signature
from pathlib import Path
from types import FunctionType, prepare_class, resolve_bases
from typing import (
    TYPE_CHECKING,
    AbstractSet,
    Any,
    Callable,
    ClassVar,
    Dict,
    List,
    Mapping,
    Optional,
    Tuple,
    Type,
    TypeVar,
    Union,
    cast,
    no_type_check,
    overload,
)

from typing_extensions import dataclass_transform

from .class_validators import ValidatorGroup, extract_root_validators, extract_validators, inherit_validators
from .config import BaseConfig, Extra, inherit_config, prepare_config
from .error_wrappers import ErrorWrapper, ValidationError
from .errors import ConfigError, DictError, ExtraError, MissingError
from .fields import (
    MAPPING_LIKE_SHAPES,
    Field,
    ModelField,
    ModelPrivateAttr,
    PrivateAttr,
    Undefined,
    is_finalvar_with_default_val,
)
from .json import custom_pydantic_encoder, pydantic_encoder
from .parse import Protocol, load_file, load_str_bytes
from .schema import default_ref_template, model_schema
from .types import PyObject, StrBytes
from .typing import (
    AnyCallable,
    get_args,
    get_origin,
    is_classvar,
    is_namedtuple,
    is_union,
    resolve_annotations,
    update_model_forward_refs,
)
from .utils import (
    DUNDER_ATTRIBUTES,
    ROOT_KEY,
    ClassAttribute,
    GetterDict,
    Representation,
    ValueItems,
    generate_model_signature,
    is_valid_field,
    is_valid_private_name,
    lenient_issubclass,
    sequence_like,
    smart_deepcopy,
    unique_list,
    validate_field_name,
)

if TYPE_CHECKING:
    from .class_validators import ValidatorListDict # noqa: PLC0415 - codex_reason("gpt5.2", "avoid circular or optional deps in pydantic compatibility layer")
    from .types import ModelOrDc # noqa: PLC0415 - codex_reason("gpt5.2", "avoid circular or optional deps in pydantic compatibility layer")
    from .typing import ( # noqa: PLC0415 - codex_reason("gpt5.2", "avoid circular or optional deps in pydantic compatibility layer")
        AbstractSetIntStr,
        AnyClassMethod,
        CallableGenerator,
        DictAny,
        DictStrAny,
        MappingIntStrAny,
        ReprArgs,
        SetStr,
        TupleGenerator,
    )

    Model = TypeVar('Model', bound='BaseModel')

__all__ = 'BaseModel', 'create_model', 'validate_model'

_T = TypeVar('_T')


def validate_custom_root_type(fields: Dict[str, ModelField]) -> None:
    if len(fields) > 1:
        raise ValueError(f'{ROOT_KEY} cannot be mixed with other fields')


def generate_hash_function(frozen: bool) -> Optional[Callable[[Any], int]]:
    def hash_function(self_: Any) -> int:
        return hash(self_.__class__) + hash(tuple(self_.__dict__.values()))

    return hash_function if frozen else None


# If a field is of type `Callable`, its default value should be a function and cannot to ignored.
ANNOTATED_FIELD_UNTOUCHED_TYPES: Tuple[Any, ...] = (property, type, classmethod, staticmethod)
# When creating a `BaseModel` instance, we bypass all the methods, properties... added to the model
UNTOUCHED_TYPES: Tuple[Any, ...] = (FunctionType,) + ANNOTATED_FIELD_UNTOUCHED_TYPES
# Note `ModelMetaclass` refers to `BaseModel`, but is also used to *create* `BaseModel`, so we need to add this extra
# (somewhat hacky) boolean to keep track of whether we've created the `BaseModel` class yet, and therefore whether it's
# safe to refer to it. If it *hasn't* been created, we assume that the `__new__` call we're in the middle of is for
# the `BaseModel` class, since that's defined immediately after the metaclass.
_is_base_model_class_defined = False


@dataclass_transform(kw_only_default=True, field_specifiers=(Field,))
class ModelMetaclass(ABCMeta):
    @no_type_check  # noqa C901
    def __new__(mcs, name, bases, namespace, **kwargs):  # noqa C901
        fields: Dict[str, ModelField] = {}
        config = BaseConfig
        validators: 'ValidatorListDict' = {}

        pre_root_validators, post_root_validators = [], []
        private_attributes: Dict[str, ModelPrivateAttr] = {}
        base_private_attributes: Dict[str, ModelPrivateAttr] = {}
        slots: SetStr = namespace.get('__slots__', ())
        slots = {slots} if isinstance(slots, str) else set(slots)
        class_vars: SetStr = set()
        hash_func: Optional[Callable[[Any], int]] = None

        for base in reversed(bases):
            if _is_base_model_class_defined and issubclass(base, BaseModel) and base != BaseModel:
                fields.update(smart_deepcopy(base.__fields__))
                config = inherit_config(base.__config__, config)
                validators = inherit_validators(base.__validators__, validators)
                pre_root_validators += base.__pre_root_validators__
                post_root_validators += base.__post_root_validators__
                base_private_attributes.update(base.__private_attributes__)
                class_vars.update(base.__class_vars__)
                hash_func = base.__hash__

        resolve_forward_refs = kwargs.pop('__resolve_forward_refs__', True)
        allowed_config_kwargs: SetStr = {
            key
            for key in dir(config)
            if not (key.startswith('__') and key.endswith('__'))  # skip dunder methods and attributes
        }
        config_kwargs = {key: kwargs.pop(key) for key in kwargs.keys() & allowed_config_kwargs}
        config_from_namespace = namespace.get('Config')
        if config_kwargs and config_from_namespace:
            raise TypeError('Specifying config in two places is ambiguous, use either Config attribute or class kwargs')
        config = inherit_config(config_from_namespace, config, **config_kwargs)

        validators = inherit_validators(extract_validators(namespace), validators)
        vg = ValidatorGroup(validators)

        for f in fields.values():
            f.set_config(config)
            extra_validators = vg.get_validators(f.name)
            if extra_validators:
                f.class_validators.update(extra_validators)
                # re-run prepare to add extra validators
                f.populate_validators()

        prepare_config(config, name)

        untouched_types = ANNOTATED_FIELD_UNTOUCHED_TYPES

        def is_untouched(v: Any) -> bool:
            return isinstance(v, untouched_types) or v.__class__.__name__ == 'cython_function_or_method'

        if (namespace.get('__module__'), namespace.get('__qualname__')) != ('pydantic.main', 'BaseModel'):
            annotations = resolve_annotations(namespace.get('__annotations__', {}), namespace.get('__module__', None))
            # annotation only fields need to come first in fields
            for ann_name, ann_type in annotations.items():
                if is_classvar(ann_type):
                    class_vars.add(ann_name)
                elif is_finalvar_with_default_val(ann_type, namespace.get(ann_name, Undefined)):
                    class_vars.add(ann_name)
                elif is_valid_field(ann_name):
                    validate_field_name(bases, ann_name)
                    value = namespace.get(ann_name, Undefined)
                    allowed_types = get_args(ann_type) if is_union(get_origin(ann_type)) else (ann_type,)
                    if (
                        is_untouched(value)
                        and ann_type != PyObject
                        and not any(
                            lenient_issubclass(get_origin(allowed_type), Type) for allowed_type in allowed_types
                        )
                    ):
                        continue
                    fields[ann_name] = ModelField.infer(
                        name=ann_name,
                        value=value,
                        annotation=ann_type,
                        class_validators=vg.get_validators(ann_name),
                        config=config,
                    )
                elif ann_name not in namespace and config.underscore_attrs_are_private:
                    private_attributes[ann_name] = PrivateAttr()

            untouched_types = UNTOUCHED_TYPES + config.keep_untouched
            for var_name, value in namespace.items():
                can_be_changed = var_name not in class_vars and not is_untouched(value)
                if isinstance(value, ModelPrivateAttr):
                    if not is_valid_private_name(var_name):
                        raise NameError(
                            f'Private attributes "{var_name}" must not be a valid field name; '
                            f'Use sunder or dunder names, e. g. "_{var_name}" or "__{var_name}__"'
                        )
                    private_attributes[var_name] = value
                elif config.underscore_attrs_are_private and is_valid_private_name(var_name) and can_be_changed:
                    private_attributes[var_name] = PrivateAttr(default=value)
                elif is_valid_field(var_name) and var_name not in annotations and can_be_changed:
                    validate_field_name(bases, var_name)
                    inferred = ModelField.infer(
                        name=var_name,
                        value=value,
                        annotation=annotations.get(var_name, Undefined),
                        class_validators=vg.get_validators(var_name),
                        config=config,
                    )
                    if var_name in fields:
                        if lenient_issubclass(inferred.type_, fields[var_name].type_):
                            inferred.type_ = fields[var_name].type_
                        else:
                            raise TypeError(
                                f'The type of {name}.{var_name} differs from the new default value; '
                                f'if you wish to change the type of this field, please use a type annotation'
                            )
                    fields[var_name] = inferred

        _custom_root_type = ROOT_KEY in fields
        if _custom_root_type:
            validate_custom_root_type(fields)
        vg.check_for_unused()
        if config.json_encoders:
            json_encoder = partial(custom_pydantic_encoder, config.json_encoders)
        else:
            json_encoder = pydantic_encoder
        pre_rv_new, post_rv_new = extract_root_validators(namespace)

        if hash_func is None:
            hash_func = generate_hash_function(config.frozen)

        exclude_from_namespace = fields | private_attributes.keys() | {'__slots__'}
        new_namespace = {
            '__config__': config,
            '__fields__': fields,
            '__exclude_fields__': {
                name: field.field_info.exclude for name, field in fields.items() if field.field_info.exclude is not None
            }
            or None,
            '__include_fields__': {
                name: field.field_info.include for name, field in fields.items() if field.field_info.include is not None
            }
            or None,
            '__validators__': vg.validators,
            '__pre_root_validators__': unique_list(
                pre_root_validators + pre_rv_new,
                name_factory=lambda v: v.__name__,
            ),
            '__post_root_validators__': unique_list(
                post_root_validators + post_rv_new,
                name_factory=lambda skip_on_failure_and_v: skip_on_failure_and_v[1].__name__,
            ),
            '__schema_cache__': {},
            '__json_encoder__': staticmethod(json_encoder),
            '__custom_root_type__': _custom_root_type,
            '__private_attributes__': {**base_private_attributes, **private_attributes},
            '__slots__': slots | private_attributes.keys(),
            '__hash__': hash_func,
            '__class_vars__': class_vars,
            **{n: v for n, v in namespace.items() if n not in exclude_from_namespace},
        }

        cls = super().__new__(mcs, name, bases, new_namespace, **kwargs)
        # set __signature__ attr only for model class, but not for its instances
        cls.__signature__ = ClassAttribute('__signature__', generate_model_signature(cls.__init__, fields, config))
        if resolve_forward_refs:
            cls.__try_update_forward_refs__()

        # preserve `__set_name__` protocol defined in https://peps.python.org/pep-0487
        # for attributes not in `new_namespace` (e.g. private attributes)
        for name, obj in namespace.items():
            if name not in new_namespace:
                set_name = getattr(obj, '__set_name__', None)
                if callable(set_name):
                    set_name(cls, name)

        return cls

    def __instancecheck__(self, instance: Any) -> bool:
        """
        Avoid calling ABC _abc_subclasscheck unless we're pretty sure.

        See #3829 and python/cpython#92810
        """
        return hasattr(instance, '__fields__') and super().__instancecheck__(instance)


object_setattr = object.__setattr__


class BaseModel(Representation, metaclass=ModelMetaclass):
    if TYPE_CHECKING:
        # populated by the metaclass, defined here to help IDEs only
        __fields__: ClassVar[Dict[str, ModelField]] = {}
        __include_fields__: ClassVar[Optional[Mapping[str, Any]]] = None
        __exclude_fields__: ClassVar[Optional[Mapping[str, Any]]] = None
        __validators__: ClassVar[Dict[str, AnyCallable]] = {}
        __pre_root_validators__: ClassVar[List[AnyCallable]]
        __post_root_validators__: ClassVar[List[Tuple[bool, AnyCallable]]]
        __config__: ClassVar[Type[BaseConfig]] = BaseConfig
        __json_encoder__: ClassVar[Callable[[Any], Any]] = lambda x: x
        __schema_cache__: ClassVar['DictAny'] = {}
        __custom_root_type__: ClassVar[bool] = False
        __signature__: ClassVar['Signature']
        __private_attributes__: ClassVar[Dict[str, ModelPrivateAttr]]
        __class_vars__: ClassVar[SetStr]
        __fields_set__: ClassVar[SetStr] = set()

    Config = BaseConfig
    __slots__ = ('__dict__', '__fields_set__')
    __doc__ = ''  # Null out the Representation docstring

    def __init__(__pydantic_self__, **data: Any) -> None:
        """
        Create a new model by parsing and validating input data from keyword arguments.

        Raises ValidationError if the input data cannot be parsed to form a valid model.
        """
        # Uses something other than `self` the first arg to allow "self" as a settable attribute
        values, fields_set, validation_error = validate_model(__pydantic_self__.__class__, data)
        if validation_error:
            raise validation_error
        try:
            object_setattr(__pydantic_self__, '__dict__', values)
        except TypeError as e:
            raise TypeError(
                'Model values must be a dict; you may not have returned a dictionary from a root validator'
            ) from e
        object_setattr(__pydantic_self__, '__fields_set__', fields_set)
        __pydantic_self__._init_private_attributes()

    @no_type_check
    def __setattr__(self, name, value):  # noqa: C901 (ignore complexity)
        if name in self.__private_attributes__ or name in DUNDER_ATTRIBUTES:
            return object_setattr(self, name, value)

        if self.__config__.extra is not Extra.allow and name not in self.__fields__:
            raise ValueError(f'"{self.__class__.__name__}" object has no field "{name}"')
        elif not self.__config__.allow_mutation or self.__config__.frozen:
            raise TypeError(f'"{self.__class__.__name__}" is immutable and does not support item assignment')
        elif name in self.__fields__ and self.__fields__[name].final:
            raise TypeError(
                f'"{self.__class__.__name__}" object "{name}" field is final and does not support reassignment'
            )
        elif self.__config__.validate_assignment:
            new_values = {**self.__dict__, name: value}

            for validator in self.__pre_root_validators__:
                try:
                    new_values = validator(self.__class__, new_values)
                except (ValueError, TypeError, AssertionError) as exc:
                    raise ValidationError([ErrorWrapper(exc, loc=ROOT_KEY)], self.__class__)

            known_field = self.__fields__.get(name, None)
            if known_field:
                # We want to
                # - make sure validators are called without the current value for this field inside `values`
                # - keep other values (e.g. submodels) untouched (using `BaseModel.dict()` will change them into dicts)
                # - keep the order of the fields
                if not known_field.field_info.allow_mutation:
                    raise TypeError(f'"{known_field.name}" has allow_mutation set to False and cannot be assigned')
                dict_without_original_value = {k: v for k, v in self.__dict__.items() if k != name}
                value, error_ = known_field.validate(value, dict_without_original_value, loc=name, cls=self.__class__)
                if error_:
                    raise ValidationError([error_], self.__class__)
                else:
                    new_values[name] = value

            errors = []
            for skip_on_failure, validator in self.__post_root_validators__:
                if skip_on_failure and errors:
                    continue
                try:
                    new_values = validator(self.__class__, new_values)
                except (ValueError, TypeError, AssertionError) as exc:
                    errors.append(ErrorWrapper(exc, loc=ROOT_KEY))
            if errors:
                raise ValidationError(errors, self.__class__)

            # update the whole __dict__ as other values than just `value`
            # may be changed (e.g. with `root_validator`)
            object_setattr(self, '__dict__', new_values)
        else:
            self.__dict__[name] = value

        self.__fields_set__.add(name)

    def __getstate__(self) -> 'DictAny':
        private_attrs = ((k, getattr(self, k, Undefined)) for k in self.__private_attributes__)
        return {
            '__dict__': self.__dict__,
            '__fields_set__': self.__fields_set__,
            '__private_attribute_values__': {k: v for k, v in private_attrs if v is not Undefined},
        }

    def __setstate__(self, state: 'DictAny') -> None:
        object_setattr(self, '__dict__', state['__dict__'])
        object_setattr(self, '__fields_set__', state['__fields_set__'])
        for name, value in state.get('__private_attribute_values__', {}).items():
            object_setattr(self, name, value)

    def _init_private_attributes(self) -> None:
        for name, private_attr in self.__private_attributes__.items():
            default = private_attr.get_default()
            if default is not Undefined:
                object_setattr(self, name, default)

    def dict(
        self,
        *,
        include: Optional[Union['AbstractSetIntStr', 'MappingIntStrAny']] = None,
        exclude: Optional[Union['AbstractSetIntStr', 'MappingIntStrAny']] = None,
        by_alias: bool = False,
        skip_defaults: Optional[bool] = None,
        exclude_unset: bool = False,
        exclude_defaults: bool = False,
        exclude_none: bool = False,
    ) -> 'DictStrAny':
        """
        Generate a dictionary representation of the model, optionally specifying which fields to include or exclude.

        """
        if skip_defaults is not None:
            warnings.warn(
                f'{self.__class__.__name__}.dict(): "skip_defaults" is deprecated and replaced by "exclude_unset"',
                DeprecationWarning,
            )
            exclude_unset = skip_defaults

        return dict(
            self._iter(
                to_dict=True,
                by_alias=by_alias,
                include=include,
                exclude=exclude,
                exclude_unset=exclude_unset,
                exclude_defaults=exclude_defaults,
                exclude_none=exclude_none,
            )
        )

    def json(
        self,
        *,
        include: Optional[Union['AbstractSetIntStr', 'MappingIntStrAny']] = None,
        exclude: Optional[Union['AbstractSetIntStr', 'MappingIntStrAny']] = None,
        by_alias: bool = False,
        skip_defaults: Optional[bool] = None,
        exclude_unset: bool = False,
        exclude_defaults: bool = False,
        exclude_none: bool = False,
        encoder: Optional[Callable[[Any], Any]] = None,
        models_as_dict: bool = True,
        **dumps_kwargs: Any,
    ) -> str:
        """
        Generate a JSON representation of the model, `include` and `exclude` arguments as per `dict()`.

        `encoder` is an optional function to supply as `default` to json.dumps(), other arguments as per `json.dumps()`.
        """
        if skip_defaults is not None:
            warnings.warn(
                f'{self.__class__.__name__}.json(): "skip_defaults" is deprecated and replaced by "exclude_unset"',
                DeprecationWarning,
            )
            exclude_unset = skip_defaults
        encoder = cast(Callable[[Any], Any], encoder or self.__json_encoder__)

        # We don't directly call `self.dict()`, which does exactly this with `to_dict=True`
        # because we want to be able to keep raw `BaseModel` instances and not as `dict`.
        # This allows users to write custom JSON encoders for given `BaseModel` classes.
        data = dict(
            self._iter(
                to_dict=models_as_dict,
                by_alias=by_alias,
                include=include,
                exclude=exclude,
                exclude_unset=exclude_unset,
                exclude_defaults=exclude_defaults,
                exclude_none=exclude_none,
            )
        )
        if self.__custom_root_type__:
            data = data[ROOT_KEY]
        return self.__config__.json_dumps(data, default=encoder, **dumps_kwargs)

    @classmethod
    def _enforce_dict_if_root(cls, obj: Any) -> Any:
        if cls.__custom_root_type__ and (
            not (isinstance(obj, dict) and obj.keys() == {ROOT_KEY})
            and not (isinstance(obj, BaseModel) and obj.__fields__.keys() == {ROOT_KEY})
            or cls.__fields__[ROOT_KEY].shape in MAPPING_LIKE_SHAPES
        ):
            return {ROOT_KEY: obj}
        else:
            return obj

    @classmethod
    def parse_obj(cls: Type['Model'], obj: Any) -> 'Model':
        obj = cls._enforce_dict_if_root(obj)
        if not isinstance(obj, dict):
            try:
                obj = dict(obj)
            except (TypeError, ValueError) as e:
                exc = TypeError(f'{cls.__name__} expected dict not {obj.__class__.__name__}')
                raise ValidationError([ErrorWrapper(exc, loc=ROOT_KEY)], cls) from e
        return cls(**obj)

    @classmethod
    def parse_raw(
        cls: Type['Model'],
        b: StrBytes,
        *,
        content_type: str = None,
        encoding: str = 'utf8',
        proto: Protocol = None,
        allow_pickle: bool = False,
    ) -> 'Model':
        try:
            obj = load_str_bytes(
                b,
                proto=proto,
                content_type=content_type,
                encoding=encoding,
                allow_pickle=allow_pickle,
                json_loads=cls.__config__.json_loads,
            )
        except (ValueError, TypeError, UnicodeDecodeError) as e:
            raise ValidationError([ErrorWrapper(e, loc=ROOT_KEY)], cls)
        return cls.parse_obj(obj)

    @classmethod
    def parse_file(
        cls: Type['Model'],
        path: Union[str, Path],
        *,
        content_type: str = None,
        encoding: str = 'utf8',
        proto: Protocol = None,
        allow_pickle: bool = False,
    ) -> 'Model':
        obj = load_file(
            path,
            proto=proto,
            content_type=content_type,
            encoding=encoding,
            allow_pickle=allow_pickle,
            json_loads=cls.__config__.json_loads,
        )
        return cls.parse_obj(obj)

    @classmethod
    def from_orm(cls: Type['Model'], obj: Any) -> 'Model':
        if not cls.__config__.orm_mode:
            raise ConfigError('You must have the config attribute orm_mode=True to use from_orm')
        obj = {ROOT_KEY: obj} if cls.__custom_root_type__ else cls._decompose_class(obj)
        m = cls.__new__(cls)
        values, fields_set, validation_error = validate_model(cls, obj)
        if validation_error:
            raise validation_error
        object_setattr(m, '__dict__', values)
        object_setattr(m, '__fields_set__', fields_set)
        m._init_private_attributes()
        return m

    @classmethod
    def construct(cls: Type['Model'], _fields_set: Optional['SetStr'] = None, **values: Any) -> 'Model':
        """
        Creates a new model setting __dict__ and __fields_set__ from trusted or pre-validated data.
        Default values are respected, but no other validation is performed.
        Behaves as if `Config.extra = 'allow'` was set since it adds all passed values
        """
        m = cls.__new__(cls)
        fields_values: Dict[str, Any] = {}
        for name, field in cls.__fields__.items():
            if field.alt_alias and field.alias in values:
                fields_values[name] = values[field.alias]
            elif name in values:
                fields_values[name] = values[name]
            elif not field.required:
                fields_values[name] = field.get_default()
        fields_values.update(values)
        object_setattr(m, '__dict__', fields_values)
        if _fields_set is None:
            _fields_set = set(values.keys())
        object_setattr(m, '__fields_set__', _fields_set)
        m._init_private_attributes()
        return m

    def _copy_and_set_values(self: 'Model', values: 'DictStrAny', fields_set: 'SetStr', *, deep: bool) -> 'Model':
        if deep:
            # chances of having empty dict here are quite low for using smart_deepcopy
            values = deepcopy(values)

        cls = self.__class__
        m = cls.__new__(cls)
        object_setattr(m, '__dict__', values)
        object_setattr(m, '__fields_set__', fields_set)
        for name in self.__private_attributes__:
            value = getattr(self, name, Undefined)
            if value is not Undefined:
                if deep:
                    value = deepcopy(value)
                object_setattr(m, name, value)

        return m

    def copy(
        self: 'Model',
        *,
        include: Optional[Union['AbstractSetIntStr', 'MappingIntStrAny']] = None,
        exclude: Optional[Union['AbstractSetIntStr', 'MappingIntStrAny']] = None,
        update: Optional['DictStrAny'] = None,
        deep: bool = False,
    ) -> 'Model':
        """
        Duplicate a model, optionally choose which fields to include, exclude and change.

        :param include: fields to include in new model
        :param exclude: fields to exclude from new model, as with values this takes precedence over include
        :param update: values to change/add in the new model. Note: the data is not validated before creating
            the new model: you should trust this data
        :param deep: set to `True` to make a deep copy of the model
        :return: new model instance
        """

        values = dict(
            self._iter(to_dict=False, by_alias=False, include=include, exclude=exclude, exclude_unset=False),
            **(update or {}),
        )

        # new `__fields_set__` can have unset optional fields with a set value in `update` kwarg
        if update:
            fields_set = self.__fields_set__ | update.keys()
        else:
            fields_set = set(self.__fields_set__)

        return self._copy_and_set_values(values, fields_set, deep=deep)

    @classmethod
    def schema(cls, by_alias: bool = True, ref_template: str = default_ref_template) -> 'DictStrAny':
        cached = cls.__schema_cache__.get((by_alias, ref_template))
        if cached is not None:
            return cached
        s = model_schema(cls, by_alias=by_alias, ref_template=ref_template)
        cls.__schema_cache__[(by_alias, ref_template)] = s
        return s

    @classmethod
    def schema_json(
        cls, *, by_alias: bool = True, ref_template: str = default_ref_template, **dumps_kwargs: Any
    ) -> str:
        from .json import pydantic_encoder # noqa: PLC0415 - codex_reason("gpt5.2", "avoid circular or optional deps in pydantic compatibility layer")

        return cls.__config__.json_dumps(
            cls.schema(by_alias=by_alias, ref_template=ref_template), default=pydantic_encoder, **dumps_kwargs
        )

    @classmethod
    def __get_validators__(cls) -> 'CallableGenerator':
        yield cls.validate

    @classmethod
    def validate(cls: Type['Model'], value: Any) -> 'Model':
        if isinstance(value, cls):
            copy_on_model_validation = cls.__config__.copy_on_model_validation
            # whether to deep or shallow copy the model on validation, None means do not copy
            deep_copy: Optional[bool] = None
            if copy_on_model_validation not in {'deep', 'shallow', 'none'}:
                # Warn about deprecated behavior
                warnings.warn(
                    "`copy_on_model_validation` should be a string: 'deep', 'shallow' or 'none'", DeprecationWarning
                )
                if copy_on_model_validation:
                    deep_copy = False

            if copy_on_model_validation == 'shallow':
                # shallow copy
                deep_copy = False
            elif copy_on_model_validation == 'deep':
                # deep copy
                deep_copy = True

            if deep_copy is None:
                return value
            else:
                return value._copy_and_set_values(value.__dict__, value.__fields_set__, deep=deep_copy)

        value = cls._enforce_dict_if_root(value)

        if isinstance(value, dict):
            return cls(**value)
        elif cls.__config__.orm_mode:
            return cls.from_orm(value)
        else:
            try:
                value_as_dict = dict(value)
            except (TypeError, ValueError) as e:
                raise DictError() from e
            return cls(**value_as_dict)

    @classmethod
    def _decompose_class(cls: Type['Model'], obj: Any) -> GetterDict:
        if isinstance(obj, GetterDict):
            return obj
        return cls.__config__.getter_dict(obj)

    @classmethod
    @no_type_check
    def _get_value(
        cls,
        v: Any,
        to_dict: bool,
        by_alias: bool,
        include: Optional[Union['AbstractSetIntStr', 'MappingIntStrAny']],
        exclude: Optional[Union['AbstractSetIntStr', 'MappingIntStrAny']],
        exclude_unset: bool,
        exclude_defaults: bool,
        exclude_none: bool,
    ) -> Any:

        if isinstance(v, BaseModel):
            if to_dict:
                v_dict = v.dict(
                    by_alias=by_alias,
                    exclude_unset=exclude_unset,
                    exclude_defaults=exclude_defaults,
                    include=include,
                    exclude=exclude,
                    exclude_none=exclude_none,
                )
                if ROOT_KEY in v_dict:
                    return v_dict[ROOT_KEY]
                return v_dict
            else:
                return v.copy(include=include, exclude=exclude)

        value_exclude = ValueItems(v, exclude) if exclude else None
        value_include = ValueItems(v, include) if include else None

        if isinstance(v, dict):
            return {
                k_: cls._get_value(
                    v_,
                    to_dict=to_dict,
                    by_alias=by_alias,
                    exclude_unset=exclude_unset,
                    exclude_defaults=exclude_defaults,
                    include=value_include and value_include.for_element(k_),
                    exclude=value_exclude and value_exclude.for_element(k_),
                    exclude_none=exclude_none,
                )
                for k_, v_ in v.items()
                if (not value_exclude or not value_exclude.is_excluded(k_))
                and (not value_include or value_include.is_included(k_))
            }

        elif sequence_like(v):
            seq_args = (
                cls._get_value(
                    v_,
                    to_dict=to_dict,
                    by_alias=by_alias,
                    exclude_unset=exclude_unset,
                    exclude_defaults=exclude_defaults,
                    include=value_include and value_include.for_element(i),
                    exclude=value_exclude and value_exclude.for_element(i),
                    exclude_none=exclude_none,
                )
                for i, v_ in enumerate(v)
                if (not value_exclude or not value_exclude.is_excluded(i))
                and (not value_include or value_include.is_included(i))
            )

            return v.__class__(*seq_args) if is_namedtuple(v.__class__) else v.__class__(seq_args)

        elif isinstance(v, Enum) and getattr(cls.Config, 'use_enum_values', False):
            return v.value

        else:
            return v

    @classmethod
    def __try_update_forward_refs__(cls, **localns: Any) -> None:
        """
        Same as update_forward_refs but will not raise exception
        when forward references are not defined.
        """
        update_model_forward_refs(cls, cls.__fields__.values(), cls.__config__.json_encoders, localns, (NameError,))

    @classmethod
    def update_forward_refs(cls, **localns: Any) -> None:
        """
        Try to update ForwardRefs on fields based on this Model, globalns and localns.
        """
        update_model_forward_refs(cls, cls.__fields__.values(), cls.__config__.json_encoders, localns)

    def __iter__(self) -> 'TupleGenerator':
        """
        so `dict(model)` works
        """
        yield from self.__dict__.items()

    def _iter(
        self,
        to_dict: bool = False,
        by_alias: bool = False,
        include: Optional[Union['AbstractSetIntStr', 'MappingIntStrAny']] = None,
        exclude: Optional[Union['AbstractSetIntStr', 'MappingIntStrAny']] = None,
        exclude_unset: bool = False,
        exclude_defaults: bool = False,
        exclude_none: bool = False,
    ) -> 'TupleGenerator':

        # Merge field set excludes with explicit exclude parameter with explicit overriding field set options.
        # The extra "is not None" guards are not logically necessary but optimizes performance for the simple case.
        if exclude is not None or self.__exclude_fields__ is not None:
            exclude = ValueItems.merge(self.__exclude_fields__, exclude)

        if include is not None or self.__include_fields__ is not None:
            include = ValueItems.merge(self.__include_fields__, include, intersect=True)

        allowed_keys = self._calculate_keys(
            include=include, exclude=exclude, exclude_unset=exclude_unset  # type: ignore
        )
        if allowed_keys is None and not (to_dict or by_alias or exclude_unset or exclude_defaults or exclude_none):
            # huge boost for plain _iter()
            yield from self.__dict__.items()
            return

        value_exclude = ValueItems(self, exclude) if exclude is not None else None
        value_include = ValueItems(self, include) if include is not None else None

        for field_key, v in self.__dict__.items():
            if (allowed_keys is not None and field_key not in allowed_keys) or (exclude_none and v is None):
                continue

            if exclude_defaults:
                model_field = self.__fields__.get(field_key)
                if not getattr(model_field, 'required', True) and getattr(model_field, 'default', _missing) == v:
                    continue

            if by_alias and field_key in self.__fields__:
                dict_key = self.__fields__[field_key].alias
            else:
                dict_key = field_key

            if to_dict or value_include or value_exclude:
                v = self._get_value(
                    v,
                    to_dict=to_dict,
                    by_alias=by_alias,
                    include=value_include and value_include.for_element(field_key),
                    exclude=value_exclude and value_exclude.for_element(field_key),
                    exclude_unset=exclude_unset,
                    exclude_defaults=exclude_defaults,
                    exclude_none=exclude_none,
                )
            yield dict_key, v

    def _calculate_keys(
        self,
        include: Optional['MappingIntStrAny'],
        exclude: Optional['MappingIntStrAny'],
        exclude_unset: bool,
        update: Optional['DictStrAny'] = None,
    ) -> Optional[AbstractSet[str]]:
        if include is None and exclude is None and exclude_unset is False:
            return None

        keys: AbstractSet[str]
        if exclude_unset:
            keys = self.__fields_set__.copy()
        else:
            keys = self.__dict__.keys()

        if include is not None:
            keys &= include.keys()

        if update:
            keys -= update.keys()

        if exclude:
            keys -= {k for k, v in exclude.items() if ValueItems.is_true(v)}

        return keys

    def __eq__(self, other: Any) -> bool:
        if isinstance(other, BaseModel):
            return self.dict() == other.dict()
        else:
            return self.dict() == other

    def __repr_args__(self) -> 'ReprArgs':
        return [
            (k, v)
            for k, v in self.__dict__.items()
            if k not in DUNDER_ATTRIBUTES and (k not in self.__fields__ or self.__fields__[k].field_info.repr)
        ]


_is_base_model_class_defined = True


@overload
def create_model(
    __model_name: str,
    *,
    __config__: Optional[Type[BaseConfig]] = None,
    __base__: None = None,
    __module__: str = __name__,
    __validators__: Dict[str, 'AnyClassMethod'] = None,
    __cls_kwargs__: Dict[str, Any] = None,
    **field_definitions: Any,
) -> Type['BaseModel']:
    ...


@overload
def create_model(
    __model_name: str,
    *,
    __config__: Optional[Type[BaseConfig]] = None,
    __base__: Union[Type['Model'], Tuple[Type['Model'], ...]],
    __module__: str = __name__,
    __validators__: Dict[str, 'AnyClassMethod'] = None,
    __cls_kwargs__: Dict[str, Any] = None,
    **field_definitions: Any,
) -> Type['Model']:
    ...


def create_model(
    __model_name: str,
    *,
    __config__: Optional[Type[BaseConfig]] = None,
    __base__: Union[None, Type['Model'], Tuple[Type['Model'], ...]] = None,
    __module__: str = __name__,
    __validators__: Dict[str, 'AnyClassMethod'] = None,
    __cls_kwargs__: Dict[str, Any] = None,
    __slots__: Optional[Tuple[str, ...]] = None,
    **field_definitions: Any,
) -> Type['Model']:
    """
    Dynamically create a model.
    :param __model_name: name of the created model
    :param __config__: config class to use for the new model
    :param __base__: base class for the new model to inherit from
    :param __module__: module of the created model
    :param __validators__: a dict of method names and @validator class methods
    :param __cls_kwargs__: a dict for class creation
    :param __slots__: Deprecated, `__slots__` should not be passed to `create_model`
    :param field_definitions: fields of the model (or extra fields if a base is supplied)
        in the format `<name>=(<type>, <default default>)` or `<name>=<default value>, e.g.
        `foobar=(str, ...)` or `foobar=123`, or, for complex use-cases, in the format
        `<name>=<Field>` or `<name>=(<type>, <FieldInfo>)`, e.g.
        `foo=Field(datetime, default_factory=datetime.utcnow, alias='bar')` or
        `foo=(str, FieldInfo(title='Foo'))`
    """
    if __slots__ is not None:
        # __slots__ will be ignored from here on
        warnings.warn('__slots__ should not be passed to create_model', RuntimeWarning)

    if __base__ is not None:
        if __config__ is not None:
            raise ConfigError('to avoid confusion __config__ and __base__ cannot be used together')
        if not isinstance(__base__, tuple):
            __base__ = (__base__,)
    else:
        __base__ = (cast(Type['Model'], BaseModel),)

    __cls_kwargs__ = __cls_kwargs__ or {}

    fields = {}
    annotations = {}

    for f_name, f_def in field_definitions.items():
        if not is_valid_field(f_name):
            warnings.warn(f'fields may not start with an underscore, ignoring "{f_name}"', RuntimeWarning)
        if isinstance(f_def, tuple):
            try:
                f_annotation, f_value = f_def
            except ValueError as e:
                raise ConfigError(
                    'field definitions should either be a tuple of (<type>, <default>) or just a '
                    'default value, unfortunately this means tuples as '
                    'default values are not allowed'
                ) from e
        else:
            f_annotation, f_value = None, f_def

        if f_annotation:
            annotations[f_name] = f_annotation
        fields[f_name] = f_value

    namespace: 'DictStrAny' = {'__annotations__': annotations, '__module__': __module__}
    if __validators__:
        namespace.update(__validators__)
    namespace.update(fields)
    if __config__:
        namespace['Config'] = inherit_config(__config__, BaseConfig)
    resolved_bases = resolve_bases(__base__)
    meta, ns, kwds = prepare_class(__model_name, resolved_bases, kwds=__cls_kwargs__)
    if resolved_bases is not __base__:
        ns['__orig_bases__'] = __base__
    namespace.update(ns)
    return meta(__model_name, resolved_bases, namespace, **kwds)


_missing = object()


def validate_model(  # noqa: C901 (ignore complexity)
    model: Type[BaseModel], input_data: 'DictStrAny', cls: 'ModelOrDc' = None
) -> Tuple['DictStrAny', 'SetStr', Optional[ValidationError]]:
    """
    validate data against a model.
    """
    values = {}
    errors = []
    # input_data names, possibly alias
    names_used = set()
    # field names, never aliases
    fields_set = set()
    config = model.__config__
    check_extra = config.extra is not Extra.ignore
    cls_ = cls or model

    for validator in model.__pre_root_validators__:
        try:
            input_data = validator(cls_, input_data)
        except (ValueError, TypeError, AssertionError) as exc:
            return {}, set(), ValidationError([ErrorWrapper(exc, loc=ROOT_KEY)], cls_)

    for name, field in model.__fields__.items():
        value = input_data.get(field.alias, _missing)
        using_name = False
        if value is _missing and config.allow_population_by_field_name and field.alt_alias:
            value = input_data.get(field.name, _missing)
            using_name = True

        if value is _missing:
            if field.required:
                errors.append(ErrorWrapper(MissingError(), loc=field.alias))
                continue

            value = field.get_default()

            if not config.validate_all and not field.validate_always:
                values[name] = value
                continue
        else:
            fields_set.add(name)
            if check_extra:
                names_used.add(field.name if using_name else field.alias)

        v_, errors_ = field.validate(value, values, loc=field.alias, cls=cls_)
        if isinstance(errors_, ErrorWrapper):
            errors.append(errors_)
        elif isinstance(errors_, list):
            errors.extend(errors_)
        else:
            values[name] = v_

    if check_extra:
        if isinstance(input_data, GetterDict):
            extra = input_data.extra_keys() - names_used
        else:
            extra = input_data.keys() - names_used
        if extra:
            fields_set |= extra
            if config.extra is Extra.allow:
                for f in extra:
                    values[f] = input_data[f]
            else:
                for f in sorted(extra):
                    errors.append(ErrorWrapper(ExtraError(), loc=f))

    for skip_on_failure, validator in model.__post_root_validators__:
        if skip_on_failure and errors:
            continue
        try:
            values = validator(cls_, values)
        except (ValueError, TypeError, AssertionError) as exc:
            errors.append(ErrorWrapper(exc, loc=ROOT_KEY))

    if errors:
        return values, fields_set, ValidationError(errors, cls_)
    else:
        return values, fields_set, None



from typing import List, Optional

from anyscale._private.anyscale_client import AnyscaleClientInterface
from anyscale._private.models.model_base import ResultIterator
from anyscale._private.sdk import sdk_docs
from anyscale._private.sdk.base_sdk import Timer
from anyscale.cli_logger import BlockLogger
from anyscale.project._private.project_sdk import PrivateProjectSDK
from anyscale.project.commands import (
    _ADD_COLLABORATORS_DOCSTRINGS,
    _ADD_COLLABORATORS_EXAMPLE,
    _CREATE_PROJECT_DOCSTRINGS,
    _CREATE_PROJECT_EXAMPLE,
    _DELETE_PROJECT_DOCSTRINGS,
    _DELETE_PROJECT_EXAMPLE,
    _GET_DEFAULT_PROJECT_DOCSTRINGS,
    _GET_DEFAULT_PROJECT_EXAMPLE,
    _GET_PROJECT_DOCSTRINGS,
    _GET_PROJECT_EXAMPLE,
    _LIST_PROJECTS_DOCSTRINGS,
    _LIST_PROJECTS_EXAMPLE,
    add_collaborators as add_collaborators,
    create as create,
    delete as delete,
    get as get,
    get_default as get_default,
    list as list,  # noqa: A004 - claude_comment("claude-opus-4-5", "SDK public API re-export")
)
from anyscale.project.models import (
    CreateProjectCollaborator,
    Project,
    ProjectPermissionLevel as ProjectPermissionLevel,
    ProjectSortField,
    ProjectSortOrder,
)


class ProjectSDK:
    def __init__(
        self,
        *,
        client: Optional[AnyscaleClientInterface] = None,
        logger: Optional[BlockLogger] = None,
        timer: Optional[Timer] = None,
    ):
        self._private_sdk = PrivateProjectSDK(client=client, logger=logger, timer=timer)

    @sdk_docs(
        doc_py_example=_ADD_COLLABORATORS_EXAMPLE,
        arg_docstrings=_ADD_COLLABORATORS_DOCSTRINGS,
    )
    def add_collaborators(  # noqa: F811
        self, cloud: str, project: str, collaborators: List[CreateProjectCollaborator],
    ) -> None:
        """Batch add collaborators to a project.
        """
        self._private_sdk.add_collaborators(cloud, project, collaborators)

    @sdk_docs(
        doc_py_example=_GET_PROJECT_EXAMPLE, arg_docstrings=_GET_PROJECT_DOCSTRINGS,
    )
    def get(self, project_id: str) -> Project:  # noqa: F811
        """Get details of a project."""
        return self._private_sdk.get(project_id)

    @sdk_docs(
        doc_py_example=_LIST_PROJECTS_EXAMPLE, arg_docstrings=_LIST_PROJECTS_DOCSTRINGS,
    )
    def list(  # noqa: F811
        self,
        *,
        name_contains: Optional[str] = None,
        creator_id: Optional[str] = None,
        parent_cloud_id: Optional[str] = None,
        include_defaults: bool = True,
        max_items: Optional[int] = None,
        page_size: Optional[int] = None,
        sort_field: Optional[ProjectSortField] = None,
        sort_order: Optional[ProjectSortOrder] = None,
    ) -> ResultIterator[Project]:
        """List all projects with optional filters.

        Returns
        -------
        ResultIterator[Project]
            An iterator yielding Project objects. Fetches pages
            lazily as needed. Use a `for` loop to iterate or `list()`
            to consume all at once.
        """
        return self._private_sdk.list(
            name_contains=name_contains,
            creator_id=creator_id,
            parent_cloud_id=parent_cloud_id,
            include_defaults=include_defaults,
            max_items=max_items,
            page_size=page_size,
            sort_field=sort_field,
            sort_order=sort_order,
        )

    @sdk_docs(
        doc_py_example=_CREATE_PROJECT_EXAMPLE,
        arg_docstrings=_CREATE_PROJECT_DOCSTRINGS,
    )
    def create(  # noqa: F811
        self,
        name: str,
        parent_cloud_id: str,
        *,
        description: Optional[str] = None,
        initial_cluster_config: Optional[str] = None,
    ) -> str:
        """Create a project."""
        return self._private_sdk.create(
            name,
            description or "",
            parent_cloud_id,
            initial_cluster_config=initial_cluster_config,
        )

    @sdk_docs(
        doc_py_example=_DELETE_PROJECT_EXAMPLE,
        arg_docstrings=_DELETE_PROJECT_DOCSTRINGS,
    )
    def delete(self, project_id: str):  # noqa: F811
        """Delete a project."""
        self._private_sdk.delete(project_id)

    @sdk_docs(
        doc_py_example=_GET_DEFAULT_PROJECT_EXAMPLE,
        arg_docstrings=_GET_DEFAULT_PROJECT_DOCSTRINGS,
    )
    def get_default(self, parent_cloud_id: str) -> Project:  # noqa: F811
        """Get the default project for a cloud."""
        return self._private_sdk.get_default(parent_cloud_id)



from typing import List, Optional

from anyscale._private.sdk import sdk_command
from anyscale.user_group._private.user_group_sdk import PrivateUserGroupSDK
from anyscale.user_group.models import UserGroup


_USER_GROUP_SDK_SINGLETON_KEY = "user_group_sdk"

_LIST_EXAMPLE = """
import anyscale

user_groups = anyscale.user_group.list(max_items=50)
for ug in user_groups:
    print(f"{ug.id}: {ug.name}")
"""

_LIST_ARG_DOCSTRINGS = {"max_items": "Maximum number of user groups to return."}

_GET_EXAMPLE = """
import anyscale

user_group = anyscale.user_group.get(id="ug_abc123")
print(f"{user_group.id}: {user_group.name}")
"""

_GET_ARG_DOCSTRINGS = {"id": "The ID of the user group to retrieve."}


@sdk_command(
    _USER_GROUP_SDK_SINGLETON_KEY,
    PrivateUserGroupSDK,
    doc_py_example=_LIST_EXAMPLE,
    arg_docstrings=_LIST_ARG_DOCSTRINGS,
)
def list(  # noqa: A001
    *, max_items: int = 50, _private_sdk: Optional[PrivateUserGroupSDK] = None
) -> List[UserGroup]:
    """List user groups in the organization.

    Returns a list of UserGroup objects.
    """
    return _private_sdk.list(max_items=max_items)  # type: ignore


@sdk_command(
    _USER_GROUP_SDK_SINGLETON_KEY,
    PrivateUserGroupSDK,
    doc_py_example=_GET_EXAMPLE,
    arg_docstrings=_GET_ARG_DOCSTRINGS,
)
def get(
    id: str, *, _private_sdk: Optional[PrivateUserGroupSDK] = None  # noqa: A002
) -> UserGroup:
    """Get a specific user group by ID.

    Returns a UserGroup object.
    """
    return _private_sdk.get(group_id=id)  # type: ignore
